@@ -16,17 +16,19 @@
  
   
 Copyright 2002 Peter Beerli and Joseph Felsenstein
+Copyright 2003-2008 Peter Beerli
+Copyright 2009 Peter Beerli and Michal Palczewski
  
-$Id: definitions.h 1458 2008-11-28 02:38:18Z beerli $
+$Id: definitions.h 1542 2009-05-26 14:04:25Z beerli $
  
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "3.0.2"
+#define MIGRATEVERSION "3.0.7"
 #else
 #ifdef SGI
 #undef MIGRATEVERSION
-#define MIGRATEVERSION "3.0.2"
+#define MIGRATEVERSION "3.0.7"
 #endif
 #endif
 #ifndef MIGRATESUBVERSION
@@ -476,6 +478,9 @@ long chains is used */
 #define WARM                   4 /*+ */
 #define HOT                    7 /*+ */
 #define VERYHOT                10 /*+ */
+#define NOTADAPTIVE            0 /*static heating scheme*/
+#define STANDARD               1 /*adaptive heating scheme*/
+#define BOUNDED                2 /*bounded adaptive scheme*/
 /* maximization material*/
 #define SINGLELOCUS            0 /*+ */
 #define MULTILOCUS             1 /*+ */
