@@ -11,7 +11,7 @@ This software is distributed free of charge for non-commercial use
 and is copyrighted. Of course, we do not guarantee that the software
 works and are not responsible for any damage you may cause or have.
  
-$Id: heating.c 1049 2007-11-29 22:47:41Z beerli $
+$Id: heating.c 1489 2009-01-03 23:47:16Z beerli $
  
 */
 /*! \file heating.c 
@@ -23,6 +23,11 @@ Heating handler for threaded heating
 #include "migration.h"
 #include "mcmc.h"
 #include "sighandler.h"
+#ifdef mPI
+#include "migrate_mpi.h"
+#endif
+
+extern int myID; //identifier for nodes in MPI context
 
 #if PTHREADS   /* ==================threaded version======== */
 
@@ -346,11 +351,65 @@ void adjust_temperatures(world_fmt ** universe, long hchains, long step, long st
             }
             universe[0]->heat = 1.;
             universe[0]->treeswapcount=0;
+	    for(i=1; i< hchains; i++)
+	      {
+		universe[i]->heat = 1./(1./universe[i-1]->heat + delta[i-1]);
+		universe[i]->treeswapcount=0;
+	      }
+            myfree(delta);
+        }
+    }
+}
+///
+/// adjust temperatures using a lower and upper bound (temperature=1 and temperature=highest)
+void adjust_temperatures_bounded(world_fmt ** universe, long hchains, long step, long steps)
+{
+  //const MYREAL corrsum = steps / HEATCHECKINTERVAL;
+    long i;
+    long deltasum;
+    MYREAL negheat = 0.0;
+    MYREAL *delta;
+    if(step == 0)
+    {
+        for(i=0; i< hchains; i++)
+        {
+            universe[i]->treeswapcount=0;
+            if(steps < HEATCHECKINTERVAL)
+                universe[i]->averageheat  = 1./universe[i]->heat;
+            else
+                universe[i]->averageheat  = 0.0;
+        }
+    }
+    else
+    {
+        if ( (step % HEATCHECKINTERVAL ) == 0)
+        {
+            universe[0]->averageheat= 1.0;
+            delta = (MYREAL *) mycalloc(hchains,sizeof(MYREAL));
+#ifdef DEBUG
+	    fprintf(stdout,"\n%i> chain 1: %f %f %li\n",myID, 1./universe[0]->heat,universe[0]->averageheat, universe[0]->treeswapcount);
+#endif
+	    deltasum = 0.0;
             for(i=1; i< hchains; i++)
             {
-                universe[i]->heat = 1./(1./universe[i-1]->heat + delta[i-1]);
-                universe[i]->treeswapcount=0;
+	      universe[i]->averageheat += HEATCHECKINTERVAL * (1./universe[i]->heat - universe[i]->averageheat) / step;
+#ifdef DEBUG
+	      fprintf(stdout,"%i> chain %li: %f %f %li (step=%li (%f))\n", myID, i, 1./universe[i]->heat,universe[i]->averageheat, universe[i]->treeswapcount, step, (MYREAL) HEATCHECKINTERVAL/step);
+#endif
+                delta[i-1] = (MYREAL) universe[i]->treeswapcount;
+		if(delta[i-1] < 1.0)
+		  delta[i-1] = 1.0;
+		deltasum += delta[i-1];
             }
+	    
+            universe[0]->heat = 1.;
+	    universe[0]->treeswapcount=0;
+            for(i=1; i < hchains-1; i++)
+            {
+	      negheat += delta[i-1]/deltasum;
+	      universe[i]->heat = 1. - negheat;
+	      universe[i]->treeswapcount=0;
+	    }
             myfree(delta);
         }
     }
