@@ -21,6 +21,7 @@ void read_from_bayesmdim_minimal_info(FILE *mdimfile, world_fmt *world,option_fm
   long tmp;
   boolean done=FALSE;
   boolean recordedusem=TRUE;
+
   input = (char *) mycalloc(LINESIZE , sizeof(char));
   while(done==FALSE && FGETS(input,LINESIZE,mdimfile) != EOF)
     {
@@ -161,11 +162,12 @@ void read_bayes_fromfile(FILE *mdimfile, world_fmt *world,option_fmt *options)
   while(FGETS(input,SUPERLINESIZE,mdimfile) != EOF)
     {
       // grab the commentlines
-      while(input[0]=='#')
+      while(input[0]=='#' || input[0]=='S')
 	{
 	  FGETS(input,LINESIZE,mdimfile);
 	}
       // read the bayesallfile
+      
       if(input !=NULL)
 	{
 	  inptr = input;
@@ -262,22 +264,27 @@ void read_bayes_fromfile(FILE *mdimfile, world_fmt *world,option_fmt *options)
 #endif
 	}
     }
-  // reset the archiving machinery
-  memset(world->auto_archive,0, sizeof(MYREAL) * 2 * world->numpop2+options->bayesmurates * world->loci + 1);
-  nnn = 1;
-  for(j=0;j<world->loci;j++)
+#ifdef BFDEBUG
+  if(world->options->datatype == 'g')
     {
-      
-      for(t=0;t<nn; t++)
+      // reset the archiving machinery
+      memset(world->auto_archive,0, sizeof(MYREAL) * 2 * world->numpop2+options->bayesmurates * world->loci + 1);
+      nnn = 1;
+      for(j=0;j<world->loci;j++)
 	{
-	  // onepass mean of autocorrelation
-	  world->auto_archive[t] += (autocorrelation[j*nn + t] - world->auto_archive[t])/nnn;
-	  // summing ess values
-	  world->ess_archive[t] += ess[j*nn + t];
-	  printf("j=%li t=%li %f\n", j, t, world->ess_archive[t]);
+	  
+	  for(t=0;t<nn; t++)
+	    {
+	      // onepass mean of autocorrelation
+	      world->auto_archive[t] += (autocorrelation[j*nn + t] - world->auto_archive[t])/nnn;
+	      // summing ess values
+	      world->ess_archive[t] += ess[j*nn + t];
+	      printf("j=%li t=%li %f\n", j, t, world->ess_archive[t]);
+	    }
+	  nnn++;
 	}
-      nnn++;
     }
+#endif /*BFDEBUG*/
   myfree(params);
   myfree(autocorrelation);
   myfree(lowerbound);
