@@ -26,7 +26,7 @@
 *   works and am not responsible for any damage you may cause or have.
 *
 */
-/* $Id: main.c 1461 2008-11-28 20:31:16Z beerli $*/
+/* $Id: main.c 1564 2009-08-01 00:30:52Z beerli $*/
 
 #include "migration.h"
 #include "sighandler.h"
@@ -55,7 +55,10 @@
 #include "pretty.h"
 #endif /*PRETTY*/
 #include <stdio.h>
-
+#ifdef BEAGLE
+#include "calculator.h"
+#include "mutationmodel.h"
+#endif
 #ifdef UEP
 #include "uep.h"
 #endif
@@ -91,7 +94,7 @@ long unique_id_global;
 #ifdef  USE_MYREAL_FLOAT
 const MPI_Datatype mpisizeof = MPI_FLOAT ;
 #else
-const MPI_Datatype mpisizeof = MPI_DOUBLE ;
+const  MPI_Datatype mpisizeof = MPI_DOUBLE ;
 #endif
 #endif
 
@@ -385,11 +388,11 @@ main (int argc, char **argv)
 	read_options_master (options);
 	options->menu = set_usemenu(usemenu, options->menu);
 	print_menu_title (stdout, options);
-	get_menu (options);
+	get_menu (options, EARTH, data);
 	check_bayes_options(options);
 #ifdef MPI        
 	MYMPIBARRIER (comm_world);
-        broadcast_options_master (options);
+        broadcast_options_master (options, data);
 #endif
       }
     else
@@ -400,12 +403,12 @@ main (int argc, char **argv)
 #endif
 	options->menu = FALSE;
       }
-    
+
     //---------------------------------------------------------------------------------------------
     // data initialization and data reading phase
     usize = (options->heating ? ( MAX (1, options->heated_chains)) : 1 );
     universe = (world_fmt **) myrealloc (universe, usize * sizeof (world_fmt *));
-    
+    // opens files on all nodes    
     init_files (EARTH, data, options);
     
     if (options->writelog && myID == MASTER)
@@ -610,6 +613,21 @@ main (int argc, char **argv)
 		fprintf(EARTH->treefile,"%s", EARTH->treespace[locus]);
 	      }
 	  }
+	// if adaptive heating print a table with the average temperatures
+	//
+	if(options->heating && options->adaptiveheat!=NOTADAPTIVE)
+	  {
+	    fprintf(EARTH->outfile,"\n\n\nAverage temperatures during the run using %s\n",(options->adaptiveheat!=NOTADAPTIVE)
+		    ==STANDARD ? "standard adaptive heating scheme" : "bounded adaptive heating scheme" );
+	    fprintf(EARTH->outfile,"===========================================================================\n\n");
+	    fprintf(EARTH->outfile,"Chain Temperature\n");
+	    // locus means indicator for chain
+	    for(locus = 0 ; locus < options->heated_chains; locus++)
+	      {
+		fprintf(EARTH->outfile,"%5li %10.5f\n",locus+1,universe[locus]->averageheat);
+	      }
+	    pdf_print_averageheat(universe,options);
+	  }
 #ifdef PRETTY
 	pdf_print_end_time(&page_height);
 	// write out to PDF file
@@ -778,6 +796,41 @@ combine_loci_mpi (char type, option_fmt * options, world_fmt * world,
 #endif
 }
 
+///
+/// calculate the number populations to analyze and resize the newpop array if necessary
+long  calculate_newpop_numpop(option_fmt *options, data_fmt *data)
+{
+  long pop;
+  long i;
+  long newnumpop=0;
+  long numpop = data->numpop;
+  long temp[data->numpop];
+  //use order of populations {1,2,3,4,5,....} , do not start with zero!!
+  //reset to options->newpops={1,1,2,1,2,3,4,...}
+  if(options->newpops_numalloc < numpop)
+    {
+      options->newpops = myrealloc(options->newpops,sizeof(long)*numpop);
+      for(i=options->newpops_numalloc;i<numpop;i++)
+	{
+	  options->newpops[i] = i+1;
+	}
+      options->newpops_numalloc = numpop;
+    }
+  memcpy(temp,options->newpops,sizeof(long)*options->newpops_numalloc);
+  qsort((void *) temp, options->newpops_numalloc, sizeof(long), longcmp);
+  pop = temp[0];
+  newnumpop = 1;
+  for(i=1;i<numpop;i++)
+    {
+      if(temp[i] != pop)
+	{
+	  newnumpop++;
+	  pop = temp[i];
+	}
+    }
+  options->newpops_numpop = newnumpop;
+  return newnumpop;
+}
 
 ///
 /// runs the MCMC sampling 
@@ -801,11 +854,13 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
     color = MPI_UNDEFINED;
   
   twolongs = (long *) mycalloc (TWO, sizeof (long));
-    
+#endif
+  getseed(options);
+#ifdef MPI
   if (myID == MASTER)
     {
 #endif
-      get_data (data->infile, data, options);
+      get_data (data->infile, data, options, EARTH);
 #ifdef MPI
       //        printf("%i > finished data reading\n", myID);
       if (numcpu == 1)
@@ -822,8 +877,17 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
 #endif
   set_plot (options);
   EARTH->cold = TRUE; // this is the cold chain when we use heating, the hotter chains have FALSE
+  options->slice_sticksizes = mycalloc(data->numpop * data->numpop + 1, sizeof(MYREAL));
+  for(i=0;i<data->numpop * data->numpop;i++)
+  {
+    options->slice_sticksizes[i] = 1.0;
+  }
+  options->slice_sticksizes[i] = 1.0;
+#ifdef NEWVERSION
+  calculate_newpop_numpop(options,data);
+#endif
   init_world (EARTH, data, options);
-  set_meanmu(EARTH,options);
+  //  set_meanmu(EARTH,options);
   if(EARTH->cold)
     single_chain_var (EARTH, 0, &var, NULL, NULL);
   
@@ -843,17 +907,16 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
       //first in universe is the cold chain[EARTH]
       heating_init (universe, usize, data, options);
     }
-  getseed (options);
-  /* calculate FST values */
-#ifdef MPI
-  if(myID!=MASTER)
-    {
-      if(options->printfst || options->thetaguess==FST || options->migrguess==FST)
-	{
-	  calc_simple_param (EARTH, data);
-	}
-    }
-#endif
+
+  //#ifdef MPI
+  //  if(myID!=MASTER)
+  //  {
+  //    if(options->printfst || options->thetaguess==FST || options->migrguess==FST)
+  //	{
+  //	  calc_simple_param (EARTH, data);
+  //	}
+  // }
+  //#endif
   /* report to screen */
   if (myID == MASTER)
     {
@@ -919,6 +982,11 @@ run_sampler (option_fmt * options, data_fmt * data, world_fmt ** universe,
               &heating_pool, maxreplicate, &treefilepos, Gmax);
 
 #endif /*MPI*/
+
+#ifdef BEAGLE
+    beagle_stop(universe, usize);
+#endif
+
 #ifdef PTHREADS
     tpool_destroy (heating_pool, 1);
 #endif
@@ -1063,7 +1131,9 @@ heated_swap (world_fmt ** universe, worldoption_fmt * options)
 {
     long sm = 0, mv = 0, vw = 0;
     long r;
-
+    //debug for heating traverse issues
+    //
+    return;
     switch (r = RANDINT (0, options->heated_chains - 2))
     {
         case 2:
@@ -1331,12 +1401,23 @@ boolean analyze_oldbayesdata(world_fmt *world, option_fmt *options, data_fmt *da
   world->cold=TRUE;
   // reads some minimal information form the header of the bayesallfile
   // this only works with files written with migrate 2.5+
+#ifdef MPI
+  if(myID==MASTER)
+    {
+      warning("does not work correctly yet");
+      read_from_bayesmdim_minimal_info(world->bayesmdimfile, world, options, data);
+      read_geofile (data, options, world->numpop);
+      init_world (world, data, options);
+      read_bayes_fromfile(world->bayesmdimfile, world, options);
+    }
+#else
   read_from_bayesmdim_minimal_info(world->bayesmdimfile, world, options, data);
   read_geofile (data, options, world->numpop);
   init_world (world, data, options);
-  set_meanmu(world,options);
-
   read_bayes_fromfile(world->bayesmdimfile, world, options);
+#endif
+  //set_meanmu(world,options);
+
 #ifdef PRETTY
   pdf_master_init(world, options, data);
 #endif
@@ -1480,21 +1561,28 @@ setup_locus (long locus, world_fmt * world, option_fmt * options,
     print_menu_locus (world->options->logfile, world, locus);
   world->start = TRUE;
   buildtree (world, options, data, locus);
+#ifdef BEAGLE
+  finish_mutationmodel(world, data, options);
+  init_beagle(world,locus);
+  set_beagle_instances(world,TRUE); // sets two instances for beagle
+  set_beagle_instances(world,FALSE); // so that wee can easily MCMC
+  fill_beagle_instances(world);
+#endif
   /*  if (world->replicate == 0)
-    {
+      {
       if (strchr (SNPTYPES, options->datatype))
-        {
-	  // unlinked snps -- does this work at all
-	  if (options->datatype == 'u')
-	    {
-	      world->data->seq->endsite *= (data->seq->addon + 1);
-	    }
-	  if (options->datatype == 'n')
-	    {
-	      world->data->seq->endsite += data->seq->addon;
-	    }
-	}
-	}
+      {
+      // unlinked snps -- does this work at all
+      if (options->datatype == 'u')
+      {
+      world->data->seq->endsite *= (data->seq->addon + 1);
+      }
+      if (options->datatype == 'n')
+      {
+      world->data->seq->endsite += data->seq->addon;
+      }
+      }
+      }
   */
 
   //    if (!options->replicate
@@ -1712,7 +1800,7 @@ heating_prepare2 (world_fmt ** universe, int usize)
     for (chain = 1; chain < usize; ++chain)
     {
         universe[chain]->G = 0;
-        universe[chain]->averageheat = universe[chain]->options->adaptiveheat ?
+        universe[chain]->averageheat = universe[chain]->options->adaptiveheat!=NOTADAPTIVE ?
             0.0 : 1. / universe[chain]->heat;
     }
     for (chain = 1; chain < usize; ++chain)
@@ -1850,6 +1938,8 @@ run_one_update (world_fmt * world)
 {
   long count = 0;
   long np = world->numpop2;
+  //fprintf(stderr,"%i> locus=%li heat=%f\n",myID, world->locus, world->heat);
+  //traverse_check(crawlback (world->root->next));
   if (world->options->bayes_infer)
     {
       if(world->bayes->mu)
@@ -1909,19 +1999,31 @@ run_updates (world_fmt ** universe,
 	else
 	  {
 	    heated_swap (universe, EARTH->options);
-	    if (options->adaptiveheat)
-	      adjust_temperatures (universe, options->heated_chains,
-	    			   inc /*rement */  + step * increment,
-	    			   steps * increment);
+	    switch (options->adaptiveheat)
+	      {
+	      case STANDARD:
+		adjust_temperatures (universe, options->heated_chains,
+				     inc /*rement */  + step * increment,
+				     steps * increment);
+		break;
+	      case BOUNDED:
+		adjust_temperatures_bounded (universe, options->heated_chains,
+				     inc /*rement */  + step * increment,
+				     steps * increment);
+		break;
+	      case NOTADAPTIVE:
+	      default:
+		break;
+	      }
 	  }
     }
-    else				/* no heating */
-    {
-        run_one_update (EARTH);
-    }
+    else
+      {		/* no heating */
+	run_one_update (EARTH);
+      }
 #ifdef UEP
     if (options->uep && EARTH->in_last_chain)
-        update_uepanc (EARTH);
+      update_uepanc (EARTH);
 #endif
 }
 
@@ -2043,6 +2145,7 @@ run_chains (world_fmt ** universe,
       if (options->heating)
 	{
 	  heating_prepare2 (universe, usize);
+
 	  for(i=1; i<EARTH->options->heated_chains; i++)
 	    {
 	      burnin_chain (universe[i]);
@@ -2090,7 +2193,7 @@ run_chains (world_fmt ** universe,
 	  }
         if (EARTH->options->heating)
 	  {
-            print_heating_progress (universe, EARTH->options, increment * steps * treeupdateratio);
+            print_heating_progress (universe, EARTH->options, EARTH->options->heating_interval * increment * steps * treeupdateratio);
 	    reset_heated_accept(universe,EARTH->options->heated_chains);
 	  }
         else
@@ -2279,18 +2382,40 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
   //  MYREAL sum = 0.;
   MYREAL heat0 = 1.;
   MYREAL heat1 = 1.;
+  MYREAL heat2 = 1.0;
   MYREAL val0  = 0.;
   MYREAL val1  = 0.;
+  MYREAL val2  = 0.;
   MYREAL bfsum = 0.;
   MYREAL bfsum2 = 0.;
   MYREAL approxlsum = 0.;
   MYREAL hsum = 0.;
   MYREAL asum = 0.;
   MYREAL lsum;
+  MYREAL lsum0;
+
+  // calculate the harmonic mean score
+  for(locus=0;locus < EARTH->loci; locus++)
+    {
+      if(EARTH->data->skiploci[locus])
+	continue;
+      hsum +=  EARTH->hmscale[locus] - log (EARTH->hm[locus]);
+    }
+
   fprintf(file,"\n\n\nLog-Probability of the data given the model (marginal likelihood = log(P(D|thisModel))\n");
   fprintf(file,"--------------------------------------------------------------------\n[Use this value for Bayes factor calculations:\nBF = Exp[log(P(D|thisModel) - log(P(D|otherModel)]\nshows the support for thisModel]\n\n");
-  
-  if(options->heating)
+#ifdef PRETTY
+  if(file==EARTH->outfile)
+    pdf_bayes_factor_header(EARTH,options);
+#endif
+  if(EARTH->loci>1)
+    {
+      fprintf(file,"\n\nLocus      Raw Thermodynamic score(1a)  Bezier approximated score(1b)     Harmonic mean(2)\n");
+      fprintf(file,"------------------------------------------------------------------------------------------\n");
+      if(file==EARTH->outfile)
+	pdf_bayes_factor_rawscores_header(EARTH,options);
+    }
+  if(options->heating)//-----------------------heating
     {
       for(locus=0;locus<EARTH->loci;locus++)
 	{
@@ -2298,6 +2423,7 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	    continue;
 
 	  lsum = 0.;
+	  lsum0 = 0.;
 #ifdef MPI
 	  // the thermodynamic integration is reporting maxreplicate times too high values
 	  // warning("this should fix the thermodynamic/mpi/replicate  problem, but needs to be checked for multilocus data");
@@ -2310,18 +2436,21 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	      done=TRUE;
 	    }
 #endif
+	  // from very hot to cold, the 
 	  for(t=1; t < hc; t++)
 	    {
+	      heat2 = heat0;
+	      val2 = val0;
 #ifdef MPI
 	      heat0 = 1./options->heat[t-1] ;
 	      heat1 = 1./options->heat[t];
 #else
 	      if(options->datatype!='g')
 		{
-		  if(options->adaptiveheat)
+		  if(options->adaptiveheat!=NOTADAPTIVE)
 		    {
-		      heat0 = universe[t-1]->averageheat ;
-		      heat1 = universe[t]->averageheat;
+		      heat0 = 1./ universe[t-1]->averageheat ;
+		      heat1 = 1./ universe[t]->averageheat;
 		    }
 		  else
 		    {
@@ -2338,9 +2467,11 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	      //Simpson's rule and trapezoidal are the same when I do only have function values at a and b
 	      val0 = EARTH->bf[locus * hc + t-1];
 	      val1 = EARTH->bf[locus * hc + t];
-	      lsum += (heat0 - heat1) * (val0 + val1) * 0.5;
+	      //we keep last element to adjust for Bezier approximation
+	      lsum0 = (heat0 - heat1) * (val0 + val1) * 0.5;
+	      lsum += lsum0; 
 #ifdef DEBUG
-	      printf("%i> sum[%li]=%f temp=(%f %f) values=(%f %f)\n",myID,t,lsum,heat0,heat1,EARTH->bf[locus * hc + t-1],EARTH->bf[locus * hc + t]);
+	      //	      printf("%i> sum[%li]=%f temp=(%f %f) values=(%f %f)\n",myID,t,lsum,heat0,heat1,EARTH->bf[locus * hc + t],EARTH->bf[locus * hc + t]);
 #endif
 	    }
 	  //	  (x2 y1 - x1 y2)/(x1 - x2)
@@ -2349,59 +2480,80 @@ void print_bayesfactor(FILE *file, world_fmt **universe, option_fmt * options)
 	  // this is certainly rough, but in simulations with 3 populations one can see that with large number
 	  // of temperatures this looks reasonable, and one can approximate the integral more accurately with 
 	  // with only a few columns.
-	  //	  intercept = (heat0 * EARTH->bf[locus * hc + t-1] - heat1 * EARTH->bf[locus * hc + t-2])/(heat0 - heat1);
-	  //approxlsum = lsum + heat1 * ((EARTH->bf[locus * hc + t-1] + intercept) * 0.5);
 	  // using Bezier to approximate nice curve between the last two points to mimick the curve that
 	  // can be found with 16 or 32 heated chains, handle points are calculated using adhoc decisions
-	  // (comaprison with 32 heated chains) using 0.8 of the interval for handle_y1 and the intercept
+	  // (comparison with 32 heated chains) using 0.8 of the interval for handle_y1 and the intercept
 	  // between the first and the second last point to calculate the handle_y2 see sumbezier()
 	  // for implementation. Currently this is not tunable.
-	  approxlsum = sumbezier(100, heat1, EARTH->bf[locus * hc + t-1], 
-				 heat0, EARTH->bf[locus * hc + t-2], 
-				 1.0, EARTH->bf[locus * hc]);
-#ifdef DEBUG
-	  fprintf(stdout,"Thermo[%li]=%f (%f)\n",locus, lsum, approxlsum);
+	  //	  approxlsum = sumbezier(100, heat1, EARTH->bf[locus * hc + t-1], 
+	  //			 heat0, EARTH->bf[locus * hc + t-2], 
+	  //			 1.0, EARTH->bf[locus * hc]);
+	  approxlsum = sumbezier(100, heat1, val1, 
+				 heat0, val0, 
+				 heat2, val2);
+	  //#ifdef DEBUG
+	  //	  fprintf(stdout,"Thermo[%li]=(%f) - (%f) + (%f) = (%f)\n",locus, lsum, lsum0, approxlsum, approxlsum + lsum - lsum0);
+	  //	  #endif
+	  if(EARTH->loci>1)
+	    {
+	      fprintf(file,"  %5li%22.2f        %22.2f           %12.2f\n", locus + 1, lsum, lsum-lsum0+approxlsum, EARTH->hmscale[locus] - log (EARTH->hm[locus])); 
+#ifdef PRETTY
+	      if(file==EARTH->outfile)
+		pdf_bayes_factor_rawscores(locus, lsum, lsum-lsum0+approxlsum, EARTH->hmscale[locus] - log (EARTH->hm[locus]));
 #endif
-	  //	  bfsum += log(lsum) + EARTH->bfscale[locus];
-	  bfsum2 += approxlsum;
-
+	    }
+	  bfsum2 += approxlsum + lsum - lsum0;  
+	  
 	  bfsum += lsum; //+ EARTH->bfscale[locus];
 	}
-      //#ifdef NEWVERSION
-      fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))=%f (%f with Bezier-approximation) %s\n", bfsum, bfsum2,  
-      	      options->adaptiveheat ? "(adaptive heating -> value may most likely be wrong)" : "");
-      //#else
-      //fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))=%f %s\n", bfsum,  
-      //	      options->adaptiveheat ? "(adaptive heating -> value may be wrong)" : "");
-      //#endif
+      if(EARTH->loci>1)
+	{
+	  fprintf(file,"---------------------------------------------------------------------------------------\n");
+	  fprintf(file,"  All  %22.2f        %22.2f           %12.2f\n", bfsum, bfsum2, hsum); 
+#ifdef PRETTY
+	  if(file==EARTH->outfile)
+	    pdf_bayes_factor_rawscores(-1, bfsum, bfsum2, hsum);
+#endif
+	}
+     else 
+       {
+	 fprintf(file,"\n\n(1a) Thermodynamic integration: log(Prob(D|Model))=%f (%f with Bezier-approximation[1b]) %s\n", bfsum, bfsum2,  
+		 options->adaptiveheat!=NOTADAPTIVE ? "(adaptive heating -> value may most likely be wrong)" : "");
+	 fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n",hsum);
+	 fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n\n");
+       }
     }
-  else
+  else   // -----------------------------------------not heating
     {
-      fprintf(file,"(1) Thermodynamic integration: log(Prob(D|Model))= UNAVAILABLE because no heating was used\n");
+      for(locus=0;locus<EARTH->loci;locus++)
+	{
+	  if(EARTH->data->skiploci[locus])
+	    continue;
+	  if(EARTH->loci>1)
+	    {
+	      fprintf(file,"    %5li           ------                           ------            %12.2f\n", locus+1, EARTH->hmscale[locus] - log (EARTH->hm[locus])); 
+#ifdef PRETTY
+	      if(file==EARTH->outfile)
+		pdf_bayes_factor_rawscores_harmo(locus, EARTH->hmscale[locus] - log (EARTH->hm[locus]));
+#endif
+	    }
+	}
+      if(EARTH->loci>1)
+	{
+	  fprintf(file,"---------------------------------------------------------------------------------------\n");
+	  fprintf(file,"  All            ------                           ------            %12.2f\n", hsum); 
+#ifdef PRETTY
+	  if(file==EARTH->outfile)
+	    pdf_bayes_factor_rawscores_harmo(-1, hsum);
+#endif
+	}
+      else
+	{
+	  fprintf(file,"\n\n(1) Thermodynamic integration: log(Prob(D|Model))= UNAVAILABLE because no heating was used\n");
+	  fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n",hsum);
+	  fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n\n");
+	}
     }
-  //  printf("%i> %li %f %f",myID, 0L, EARTH->hmscale[0],EARTH->amscale[0]);
-  //printf(" %f %f\n",EARTH->hm[0],EARTH->am[0]);
-  hsum =  0.0;
-  for(locus=0;locus < EARTH->loci; locus++)
-    {
-      //  printf("%i> %li %f %f",myID, locus, EARTH->hmscale[locus],EARTH->amscale[locus]);
-      //printf(" %f %f\n",EARTH->hm[locus],EARTH->am[locus]);
-      if(EARTH->data->skiploci[locus])
-	continue;
-      //      fprintf(stdout,"    Harmonic[%li]=%f\n",locus, EARTH->hmscale[locus] - log (EARTH->hm[locus]));
-      hsum +=  EARTH->hmscale[locus] - log (EARTH->hm[locus]);
-    }
-  fprintf(file,"(2) Harmonic mean:             log(Prob(D|Model))=%f\n",hsum);
-  asum =  0.0;
-  //      for(locus=0;locus < EARTH->loci; locus++)
-  //	{
-  //	  if(EARTH->data->skiploci[locus])
-  //	    continue;
-  //	  asum += EARTH->amscale[locus]  + log (EARTH->am[locus] / (lsteps*maxreplicate));
-  //	}
-  //      fprintf(file,"(3) Arithmetic mean:           log(Prob(D|Model))=%f\n", asum);
-  fprintf(file,"(1) and (2) should give a similar result, (2) is considered more\ncrude than (1), but (1) needs heating with several well-spaced chains\n\n");
-  //(3) maybe strongly biased by large values\n\n\n");
 #ifdef PRETTY
   if(file==EARTH->outfile)
     pdf_bayes_factor(EARTH, bfsum, bfsum2, hsum, asum, maxreplicate);
@@ -2421,7 +2573,7 @@ MYREAL sumbezier(long intervals, MYREAL x0, MYREAL y0, MYREAL x1, MYREAL y1, MYR
   const MYREAL inv_interval = 1./intervals;
   const MYREAL sx0 = x0;
   const MYREAL sx1 = x0;
-  const MYREAL sy0 = 0.2 * y0 + 0.8 * y1;
+  const MYREAL sy0 = 0.2 * y0 + 0.8 * y2;
   const MYREAL sy1 = (-x2 * y1 + x1 * y2)/(x1 - x2);
   MYREAL t     = 0.0;
   MYREAL t2    = 0.0;
@@ -2438,7 +2590,7 @@ MYREAL sumbezier(long intervals, MYREAL x0, MYREAL y0, MYREAL x1, MYREAL y1, MYR
   // intialize with t=0.0
   oldx = x0;
   oldy = y0;
-  fprintf(stdout,"\n\n%f %f %f %f %f %f\n",x2,y2,sx0,sy0, x0,y0);
+  //fprintf(stdout,"\n\n%f %f %f %f %f %f\n",x2,y2,sx0,sy0, x0,y0);
   for(t=inv_interval; t <= 1.0; t += inv_interval)
     {
       onet  = 1.0 - t;
@@ -2451,12 +2603,12 @@ MYREAL sumbezier(long intervals, MYREAL x0, MYREAL y0, MYREAL x1, MYREAL y1, MYR
       //      newx = 3sx0 (1-t)^2 t + 3 sx1 (1-t) t^2 + (1-t)^3 x0 + t^3 x1
       newx = sx0 * onet2 + sx1 * t2 + onet3 * x0 + t3 * x1;
       newy = sy0 * onet2 + sy1 * t2 + onet3 * y0 + t3 * y1;
-      fprintf(stdout,"%f %f\n",newx,newy);
+      //      fprintf(stdout,"%f %f\n",newx,newy);
       sum += (newx - oldx) * (newy + oldy)/2.;
       oldx = newx;
       oldy = newy;
     }
-  fprintf(stdout,"%f %f %f %f\n\n\n",x1,y1,sx1,sy1);
+  //  fprintf(stdout,"%f %f %f %f\n\n\n",x1,y1,sx1,sy1);
   return sum;
 }
 
@@ -2536,14 +2688,14 @@ void calculate_BF(world_fmt **universe, option_fmt *options)
 }
 
 #ifdef BFDEBUG
-void  print_marginal_order(FILE *file, world_fmt *world)
+void  print_marginal_order(char *buf, long *bufsize, world_fmt *world)
 {
   long i;
 
   for(i=0;i<world->options->heated_chains;i++)
-    fprintf(file,"# --  %s = %f\n", "Thermodynamic temperature", world->options->heat[i]);
-    fprintf(file,"# --  %s\n", "Marginal log(likelihood) [Thermodynamic integration]");
-    fprintf(file,"# --  %s\n", "Marginal log(likelihood) [Harmonic mean]");
+    *bufsize += sprintf(buf+ *bufsize,"# --  %s = %f\n", "Thermodynamic temperature", world->options->heat[i]);
+  *bufsize += sprintf(buf+ *bufsize,"# --  %s\n", "Marginal log(likelihood) [Thermodynamic integration]");
+  *bufsize += sprintf(buf+ *bufsize,"# --  %s\n", "Marginal log(likelihood) [Harmonic mean]");
 }
 
 #ifdef MPI /* */
@@ -2585,7 +2737,7 @@ void      print_marginal_like(char *temp, long *c, world_fmt * world)
       lsum = 0.;
       for(t=1; t < hc; t++)
 	{
-	  if(world->options->adaptiveheat)
+	  if(world->options->adaptiveheat!=NOTADAPTIVE)
 	    {
 	      heat0 = world->options->averageheat[t-1] ;
 	      heat1 = world->options->averageheat[t];
@@ -2788,13 +2940,25 @@ void reset_bayesmdimfile(world_fmt *world, option_fmt *options)
       get_bayeshist (world, options); // this is transferring some material but
       // the parameter file is not filled in anymore [-> small data block]
       // save parameters in file and reload
+#ifdef MPI
+      // close all for bayesallfile
+#ifdef PARALIO
+      MPI_File_close(&world->mpi_bayesmdimfile);
+#endif
+#endif
       if(myID==MASTER)
 	{
+#ifdef MPI
+#ifdef PARALIO
+	  world->bayesmdimfile = fopen(options->bayesmdimfilename,"r+");
+#else
 	  fflush(world->bayesmdimfile);
-	  //	  FClose(world->bayesmdimfile);
-
 	  world->bayesmdimfile = freopen(options->bayesmdimfilename,"r+", world->bayesmdimfile);
-
+#endif
+#else
+	  fflush(world->bayesmdimfile);
+	  world->bayesmdimfile = freopen(options->bayesmdimfilename,"r+", world->bayesmdimfile);
+#endif
 	  read_bayes_fromfile(world->bayesmdimfile,world, options);
 	  for(locus=0;locus<world->loci;locus++)
 	    {
