@@ -18,7 +18,7 @@ Copyright 2003-2006 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc1.c 1464 2008-12-05 16:37:24Z beerli $
+$Id: mcmc1.c 1565 2009-08-03 14:48:56Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc1.c
 
@@ -46,6 +46,8 @@ Tree changer and acceptance rejection scheme
 #define NO_MIGR_NODES 0
 #define WITH_MIGR_NODES 1
 
+boolean debugtip;
+
 extern int myID;
 extern MYREAL probg_treetimesX(world_fmt *world, vtlist *tl, long T);
 
@@ -66,7 +68,6 @@ void construct_localtimelist (timelist_fmt * timevector,
                               proposal_fmt * proposal);
 void traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
                        long *oldnode_elem, int include_migration);
-void add_locallineages (timelist_fmt * timevector, proposal_fmt * proposal);
 void chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
                    node ** bordernodes, long *bordernum);
 void findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
@@ -96,6 +97,7 @@ void count_migrations (node * p, long *count);
 long migration_from (long to, proposal_fmt * proposal);
 
 MYREAL prob_tree (world_fmt * world, timelist_fmt * tyme);
+void traverse_check(node *theNode);
 
 /* Functions ++++++++++++++++++++++++++++++++++++++++++++++++*/
 void
@@ -184,16 +186,21 @@ tree_update (world_fmt * world, long g)
     tentry = &(*timevector).tl[0];
     age = proposal->origin->tyme;
     zz = 0;
-    while (tentry->age <= age && zz < (*timevector).T)
+    while ((tentry->age < age || tentry->age - age < SMALLEPSILON)&& zz < (*timevector).T)
       {
 	tentry = &(*timevector).tl[zz];
 	zz++;
       }
+    zz--;
     nexttime = tentry->age;
     if ((*timevector).T > 1)
       endtime = (*timevector).tl[(*timevector).T - 2].age;
     else
       endtime = 0.0;
+    proposal->time = 0.0;
+#ifdef TREEDEBUG
+    fprintf(stdout,"%i> begin zz=%li nextime=%f (pt=%f+age=%f)=%f,pot=%f\n",myID, zz, nexttime, proposal->time, age, age+proposal->time,proposal->origin->tyme);
+#endif
     proposal->time = age;
     coalesced = FALSE;
     /*------------------------------------
@@ -208,18 +215,28 @@ tree_update (world_fmt * world, long g)
 	  (proposal->migr_table_counter >
 	   0) ? proposal->migr_table[proposal->migr_table_counter -
 				     1].from : proposal->origin->pop;
-        proposal->time = age + eventtime (proposal, actualpop, tentry, &event);
-#ifdef DEBUG
-	//	if(show==1)
-	//  {
-	//	 printf("%0.5f  %3li  %c  %3li",proposal->time,actualpop,event,tentry->lineages[0]);
-	//long jj;
-	//for(jj = 1;jj < proposal->world->numpop; jj++)
-	//  printf(" %3li",tentry->lineages[jj]);
-	//printf("\n");
-	//}
+#ifdef TREEDEBUG
+	if(age == 0.0 && proposal->origin->tyme>0.0)
+	  {
+	    printf("\n\n%f\n\n",age);
+	  }
 #endif
-        //      FPRINTF(stderr,"age=%f event=%c\n",proposal->time,event);
+        proposal->time = eventtime (proposal, actualpop, tentry, &event);
+#ifdef TREEDEBUG
+	fprintf(stdout,"%i> zz=%li (pt=%f+age=%f)=%f,pot=%f\n",myID, zz, proposal->time, age, age+proposal->time,proposal->origin->tyme);
+#endif
+	proposal->time += age;
+	//	age = proposal->time;
+	
+	if(proposal->time < proposal->origin->tyme)
+	  {
+	    fprintf(stdout,"%i> Proposal failed because of unordered entry in time list, abort this sample\nProposed time=%f origin time=%f nexttime=%f\n", myID, proposal->time, proposal->origin->tyme, nexttime);
+	    // we end up here when the migration events exceed the upper limit
+	    free_proposal (proposal);
+	    free_timevector (timevector);
+	    return 0;
+	    //	    error("Proposal of time failed\n");
+	  }
         if (proposal->time < nexttime)
 	  {
             if (event == 'm')
@@ -232,7 +249,7 @@ tree_update (world_fmt * world, long g)
 		    //warning("Event was migration event but could not match a lineage to it\n");
                     return 0;
                 }
-                age = proposal->time;
+		age = proposal->time;
                 continue;
             }
             else
@@ -241,10 +258,19 @@ tree_update (world_fmt * world, long g)
                               &bordernum);
 		if(bordernum == 0)
 		  {
-		    warning("chooseTarget failed at age %f\n", proposal->time);
-		    free_proposal (proposal);
-		    free_timevector (timevector);
-		    return 0;
+		    if (!migrate (proposal, proposal->origin))
+		      {
+			// we end up here when the migration events exceed the upper limit
+			free_proposal (proposal);
+			free_timevector (timevector);
+			return 0;
+		      }
+		    age = proposal->time;
+		    continue;
+		    //		    warning("chooseTarget failed at age %f\n", proposal->time);
+		    //		    free_proposal (proposal);
+		    //		    free_timevector (timevector);
+		    //		    return 0;
 		  }
                 pretendcoalesce1p (proposal);
 #ifdef UEP
@@ -254,15 +280,22 @@ tree_update (world_fmt * world, long g)
                 coalesced = TRUE;
                 break;
             }
-        }   /*end if proposal->time < nextime */
+	  }   /*end if proposal->time < nextime */
         age = nexttime;
-        tentry = &(*timevector).tl[(tentry->slice) + 1]; /*next entry in timelist */
-	if((tentry->slice+1) >= timevector->T)
+#ifdef TREEDEBUG
+	if(age < proposal->origin->tyme)
+	  {
+	    printf("darn: nextime=%f\n",nexttime);
+	  }
+#endif
+	zz++;
+	if(zz >= timevector->T)
 	  {
 	    break;
 	  }
+	tentry = &(*timevector).tl[zz]; /*next entry in timelist */
         nexttime = tentry->age;
-    }
+      }
     if (!coalesced)
     {
       if (!pre_population(proposal, (*timevector).tl, (*timevector).T - 1, &slider))
@@ -286,11 +319,16 @@ tree_update (world_fmt * world, long g)
 #endif
         (acceptlike (world, proposal, g, timevector)))
       {
+#ifdef BEAGLE
+	printf("Accepted\n");
+#endif
 	if (proposal->time > world->root->tyme)
 	  {   /*saveguard */
             world->root->tyme += proposal->time;
 	  }
         coalesce1p (proposal);
+	//DEBUG
+	//traverse_check(crawlback (proposal->root->next));printf("*");
 #ifdef DEBUG
 	//traverse_adjust(world->root->next->back, 1.0);
 #endif
@@ -315,8 +353,15 @@ tree_update (world_fmt * world, long g)
             world->likelihood[g] = /*world->ueplikelihood +*/ world->likelihood[g];
 	  }
 #else
+#ifdef BEAGLE
+	set_beagle_dirty(proposal->origin,proposal->target,showtop(world->root->next->back));
+	reset_beagle(world->beagle);
+	smooth (world->root->next, crawlback(world->root->next), world, world->locus);
         world->likelihood[g] = treelikelihood (world);
-#endif
+#else
+        world->likelihood[g] = treelikelihood (world);
+#endif /*BEAGLE*/
+#endif /*UEP*/
         /* create a new timelist */
         construct_tymelist (world, &world->treetimes[0]);
         if (world->options->treeprint != NONE && world->cold)
@@ -335,6 +380,7 @@ tree_update (world_fmt * world, long g)
 	  {
 	    world->bayes->stoptime = -proposal->time;
 	  }
+	//traverse_check(crawlback (proposal->root->next));printf(".");
 	//
       }
     free_proposal (proposal);
@@ -499,6 +545,13 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
         (*proposal)->umf = (MYREAL *) mycalloc (world->data->uepsites, sizeof (MYREAL));
     }
 #endif
+
+#ifdef BEAGLE
+    (*proposal)->leftid   = 0;
+    (*proposal)->rightid  = 0;
+    (*proposal)->parentid = 0;
+    reset_beagle(world->beagle);
+#endif
 }
 
 ///
@@ -651,7 +704,9 @@ void traverse_tagnew(node *theNode, node *origin)
   if (theNode->type != 't'  && theNode->next->next->back != NULL)
     {
       if(theNode->top != 1)
-	error("");
+	theNode = showtop(theNode);
+      	//error("");
+
       traverse_tagnew (theNode->next->back, origin);       
       if (theNode->type != 'm' && theNode->next->next->back != NULL)
 	{
@@ -677,7 +732,9 @@ chooseOrigin (proposal_fmt * proposal)
     long elem = 0, oldelem = (proposal->sumtips * 2.);
     node *tmp=NULL, **goal;
     goal = (node **) mycalloc (oldelem, sizeof (node *));
+    //#ifdef DEBUG
     //traverse_check(crawlback (proposal->root->next));
+    //#endif
     traverseAllNodes (crawlback (proposal->root->next), &goal, &elem, &oldelem,
                       NO_MIGR_NODES);
     switch(elem)
@@ -719,14 +776,35 @@ chooseOrigin (proposal_fmt * proposal)
 void
 construct_localtimelist (timelist_fmt * timevector, proposal_fmt * proposal)
 {
-    long z = 0;
-    long oz = proposal->listsize;
-    //    long numpop = timevector->numpop = proposal->numpop;
-    traverseAllNodes (crawlback (proposal->origin)->back,
+#ifdef TREEDEBUG
+  double dif;
+  long ii;
+#endif
+  long z = 0;
+  long oz = proposal->listsize;
+  //    long numpop = timevector->numpop = proposal->numpop;
+  traverseAllNodes (crawlback (proposal->origin)->back,
                       &proposal->aboveorigin, &z, &oz, WITH_MIGR_NODES);
-    proposal->aboveorigin[z++] = proposal->oback;
-    proposal->aboveorigin[z] = NULL;
-    prune_timelist(&proposal->world->treetimes[0],timevector, proposal->aboveorigin, proposal, proposal->world->numpop);
+  proposal->aboveorigin[z++] = proposal->oback;
+  proposal->aboveorigin[z] = NULL;
+  prune_timelist(&proposal->world->treetimes[0],timevector, proposal->aboveorigin, proposal, proposal->world->numpop);
+  add_partlineages(proposal->numpop, &timevector);
+#ifdef TREEDEBUG
+  for(ii=0;ii<timevector->T-2;ii++)
+    {
+      printf("%i> ii=%li %0.5f  (%li->%li) %c %3li",myID, ii, timevector->tl[ii].age, 
+	     timevector->tl[ii].from, timevector->tl[ii].to, 
+	     timevector->tl[ii].eventnode->type,timevector->tl[ii].lineages[0]);
+      long jj;
+      for(jj = 1;jj < proposal->numpop; jj++)
+	printf(" %3li",timevector->tl[ii].lineages[jj]);
+      printf(" backtyme:%f dif:%f\n",showtop(timevector->tl[ii].eventnode->back)->tyme,dif=showtop(timevector->tl[ii].eventnode->back)->tyme-timevector->tl[ii].eventnode->tyme);
+      if(dif<0.0)
+	{
+	  error("shit");
+	}
+    }
+#endif
 }
 
 /*----------------------------------------------------------------------------
@@ -741,12 +819,13 @@ void
 traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
                   long *oldnode_elem, int include_migration)
 {
-  static long counter=0;
+    static long counter=0;
     long elem;
-    counter++;
+      counter++;
     if(theNode->type != 'r' && theNode->tyme > showtop(theNode->back)->tyme)
       {
-	printf("function was called %li times\n",counter);
+	printf("%i> TTRAVERSETRAVERSETRAVERSETRAVERSE\nfunction was called with %li times: with node (%c) time %f and node (%c) time %f\n",
+	       myID, counter, theNode->type,  theNode->tyme, showtop(theNode->back)->type, showtop(theNode->back)->tyme);
 	error("traverseAllNodes() failed");
       }
     
@@ -791,7 +870,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
 				oldnode_elem, WITH_MIGR_NODES);
             if ((*node_elem) == (*oldnode_elem - 2))
 	      {
-                elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
+                elem = (*oldnode_elem) + (*oldnode_elem);
                 (*nodelist) =
 		  (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
@@ -804,7 +883,7 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
 	  {
             if ((*node_elem) == (*oldnode_elem - 2))
 	      {
-                elem = *oldnode_elem = ((*oldnode_elem) + (*oldnode_elem) / 2);
+                elem = (*oldnode_elem) + (*oldnode_elem);
                 (*nodelist) =
 		  (node **) myrealloc ((*nodelist), sizeof (node *) * elem);
                 memset ((*nodelist) + (*oldnode_elem), 0,
@@ -823,15 +902,15 @@ traverseAllNodes (node * theNode, node *** nodelist, long *node_elem,
 void
 prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv,  register node ** __restrict ptr, proposal_fmt * proposal, long numpop)
 {
-  long i = 0;
-  register long j = 0;
-  long slot = 0;
-  //  long size = numpop * sizeof(long);
-  //  long * tmpptr = NULL;
-  register node *thenode;
-  vtlist * tls;
+  register long i    = 0;
+  register long j    = 0;
+  register node * thenode;
+  long          slot = 0;
+  vtlist        * tls;
 
     slot=0;
+    // go through all slices in old time list and
+    // extract node pointers
     for (i = 0; i < (*oldtv).T; i++)
     {
       j = 0;
@@ -840,6 +919,8 @@ prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv,  register node ** __
 	{
 	  j++;
 	}
+      // if the comparison reaches the end of the ptr list that holds an NULL element at the end
+      // then the node needs to be present in the new time list.
       if (ptr[j] == NULL)
 	{
 	  tls = &((*newtv).tl[slot]);
@@ -853,32 +934,14 @@ prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv,  register node ** __
 	}
     }
     (*newtv).T = slot;
-    if ((*newtv).tl[(*newtv).T - 1].eventnode->type != 'r')
-    {
-        error ("Root not at the end of local timelist\n");
-    }
-    add_locallineages (newtv, proposal);
+    //   if ((*newtv).tl[(*newtv).T - 1].eventnode->type != 'r')
+    //{
+    //    error ("Root not at the end of local timelist\n");
+    //}
+    // sort seems to be necessary with dated samples
+    qsort ((void *) (*newtv).tl, (*newtv).T, sizeof (vtlist), agecmp);
 }
 
-void
-add_locallineages (timelist_fmt * timevector, proposal_fmt * proposal)
-{
-    long pop = -99, numpop = proposal->world->numpop;
-    if (timevector->T <= 0)
-        error ("Help: timelist contains 0 elements");
-    for (pop = 0; pop < numpop; pop++)
-        timevector->tl[timevector->T - 1].lineages[pop] = 0;
-    if (timevector->T == 1)
-        timevector->tl[0].lineages[proposal->osister->actualpop] += 1;
-    else
-        timevector->tl[timevector->T -
-                       1].lineages[timevector->tl[timevector->T-2].from] += 1;
-
-    add_partlineages (numpop, &timevector);
-
-}
-
-
 /* replaces nodepointers in list 1 with NULL if they are present in list 2
    returns the first NULL slot in the array.
  */
@@ -914,9 +977,11 @@ xor (node ** ptrl1, node ** ptrl2)
 int
 migrate (proposal_fmt * proposal, node * up)
 {
+  long tmp;
+  long numpop = proposal->numpop;
     long i = proposal->migr_table_counter;
     migr_table_fmt *array = proposal->migr_table;
-    if (i > MIGRATION_LIMIT * proposal->numpop)
+    if (i > MIGRATION_LIMIT * numpop || numpop < 2)
     {
         return 0;
     }
@@ -924,10 +989,20 @@ migrate (proposal_fmt * proposal, node * up)
         array[i].to = array[i - 1].from;
     else
         array[i].to = up->pop;
-    array[i].from = migration_from (array[i].to, proposal);
+    tmp = migration_from (array[i].to, proposal);
+
+    if(tmp < numpop)
+      array[i].from = tmp;
+    else
+      return 0;
+
     //DEBUG
     // printf("i: %3li %li %li\n",i,proposal->migr_table[i].from,proposal->migr_table[i].to);
     array[i++].time = proposal->time;
+    if(proposal->time < proposal->origin->tyme)
+      {
+	error("in migrate() wrong time found");
+      }
     if (i >= proposal->old_migr_table_counter)
     {
         proposal->old_migr_table_counter += 10;
@@ -1080,10 +1155,16 @@ migration_from (long to, proposal_fmt * proposal)
     long ii = 0;
     MYREAL *r = proposal->world->migproblist[to];
     MYREAL rr = UNIF_RANDUM ();
-    while (rr >  r[ii])
+    while (rr >  r[ii] && ii < proposal->world->numpop-1)
     {
         ii++;
     }
+#ifdef DEBUG
+    if(r[ii] == 0.0)
+      {
+	warning("DEBUG: migproblist does not contain a migration probability\n");
+      }
+#endif
     if (ii < to)
         return ii;
     else
@@ -1103,19 +1184,15 @@ chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
         actualpop = proposal->origin->pop;
     else
         actualpop = proposal->migr_table[proposal->migr_table_counter - 1].from;
-    //if (rb->tyme < proposal->time)
-    //{
-      //        error ("Wrong Time for action in chooseTarget()\n");
-    //    proposal->target = NULL;
-    //    proposal->tsister = NULL;
-    //    proposal->realtsister = NULL;
-    //    proposal->realtarget = NULL;
-    //	return;
-    //}
+    if (rb->tyme < proposal->time)
+      {
+	error ("Wrong Time for action in chooseTarget()\n");
+      }
     findbordernodes (rb, proposal, actualpop, &bordernodes, &proposal->listsize, bordernum,
                      &(*timevector).tl, (*timevector).T);
     if (*bordernum > 0)
     {
+      // found elegible linages to coalesce to
         proposal->target = bordernodes[RANDINT (0, (*bordernum) - 1)];
         if (proposal->target != rb)
         {
@@ -1130,6 +1207,7 @@ chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
     }
     else
     {
+      // no lineage to coalesce to was found.
         proposal->target = NULL;
         proposal->tsister = NULL;
         proposal->realtsister = NULL;
@@ -1143,6 +1221,9 @@ findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
                  long gte)
 {
     node *tmp, *back;
+    // we search on the old tree and reaching the oback node that was excised from
+    // from the residual tree we jump over it and go up the sister branch and back is
+    // going further down than the oback node
     if (theNode == proposal->oback)
     {
         tmp = showtop (crawlback (proposal->osister)->back);
@@ -1156,20 +1237,29 @@ findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
     if (pop == tmp->pop && pop == back->actualpop && tmp->tyme < proposal->time
             && back->tyme > proposal->time)
     {
-      if(*bordernum >= *allocsize)
-	{
-	  warning("reallocation in finderbordernodes -- strange");
-	  (*bordernodes) = (node **) myrealloc(*bordernodes, (*allocsize+50) * sizeof(node*));
-	  memset(*bordernodes + *allocsize, 0, sizeof(node*) * 50);
-	  *allocsize = *allocsize + 50;
-	}
+      // lineage end points bracket the the new proposed time,
+      // therefore this branch needs to be included
       (*bordernodes)[(*bordernum)++] = tmp;
       return;
     }
     else
     {
+      // the lineage does not fit the populations (it may fit the time, so)
+      // the if here checks this
         if (back->tyme < proposal->time)
-            return;
+	  {
+	    // this makes sure that we looked at all lineages that
+	    // may be electable to receive a coalescent have been looked
+	    // at, if we reach here the time of the proposal is older than
+	    // all available lineages on this branch and we safely can stop
+	    // following this branch.
+	    return;
+	  }
+	// if we are still working on interior nodes and the proposal time is still 
+	// younger than then lineage bracketing nodes we need to continue our search
+	// up in the tree following right and left branches, but need to stop when 
+	// we encounter a tip node (this is important with dated tips because these can
+	// be interspersed in the timelist.
         if (tmp->type != 't')
         {
             if (tmp->next->back != NULL)
@@ -1308,6 +1398,7 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
         //mm += proposal->param0[i];
         //}
 
+      //limits treesize
       if(proposal->time > 1000000)
 	return 0;
 
@@ -1540,6 +1631,12 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
       //            report = FALSE;
       //            oldg = g;
       //        }
+#ifdef BEAGLE
+      change_beagle(world->beagle);
+      printf("Reset to: ");
+      set_beagle_dirty(proposal->origin,proposal->target,showtop(world->root->next->back));
+      calcLnL(world, world->beagle->instance);
+#endif
       return FALSE;
     }
 #ifdef UEP
@@ -1565,6 +1662,10 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
     if (world->likelihood[g] < proposal->likelihood)
     {
       not_accepted = 0;
+#ifdef BEAGLE
+      //    printf("Adjust other instance: ");
+      //calcLnL(world,!world->beagle->instance);
+#endif
       return TRUE;
     }
     if (!world->options->heating)
@@ -1574,6 +1675,10 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
         if (rr < expo)
 	  {
 	    not_accepted = 0;
+#ifdef BEAGLE
+	    //    printf("Adjust other instance: ");
+	    //calcLnL(world,!world->beagle->instance);
+#endif
             return TRUE;
 	  }
     }
@@ -1584,11 +1689,26 @@ acceptlike (world_fmt * world, proposal_fmt * proposal, long g,
         if (rr < expo)
 	  {
 	    not_accepted = 0;
+#ifdef BEAGLE
+	    //    printf("Adjust other instance: ");
+	    // calcLnL(world,!world->beagle->instance);
+#endif
             return TRUE;
 	  }
     }
     if(world->options->prioralone)
-      return TRUE;
+      {
+#ifdef BEAGLE
+	//    printf("Adjust other instance: ");
+	//calcLnL(world,!world->beagle->instance);
+#endif
+	return TRUE;
+      }
+#ifdef BEAGLE
+    change_beagle(world->beagle);
+    printf("Reset to: ");
+    calcLnL(world,world->beagle->instance);
+#endif
     return FALSE;
 }
 
@@ -1629,7 +1749,7 @@ eventtime (proposal_fmt * proposal, long pop, vtlist * tentry, char *event)
     lines    = 2.0 * tentry->lineages[pop];
     denom    = mm + (lines * (1.0 / (proposal->param0[pop]*rate)));
     invdenom = 1.0 / denom;
-    interval = (-(LOG (/*nu = */UNIF_RANDUM ())) * invdenom) ;
+    interval =  (-(LOG (/*nu = */UNIF_RANDUM ())) * invdenom) ;
     //could be used to calculate p(gn|go): proposal->nu += nu; //adds the probabilities (=random numbers);
     //[overcautious]
     //if (interval < 0.0)
