@@ -18,7 +18,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc2.c 1323 2008-07-25 19:13:48Z beerli $
+$Id: mcmc2.c 1563 2009-07-30 18:28:33Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc2.c
 Contains low-level routines to manipulate the treechanger
@@ -31,7 +31,9 @@ Contains low-level routines to manipulate the treechanger
 #include "tools.h"
 #include "mcmc.h"
 #include "migrate_mpi.h"
-
+#ifdef BEAGLE
+#include "calculator.h"
+#endif
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
@@ -190,16 +192,16 @@ coalesce1p (proposal_fmt * proposal)
     //    if(proposal->world->in_last_chain)
     traverse_tagnew(crawlback (proposal->root->next),proposal->origin);
 #ifdef MESS
-    traverse_checker(proposal->root);
-    if(skip==0)
-      {
+    //traverse_checker(proposal->root);
+    //if(skip==0)
+    //  {
 	FPRINTF (stdout,
 		 "\n[& Comment: Locus %li, best log likelihood = %f (to file),heat=%f]\n",
 		 proposal->world->locus + 1, proposal->world->likelihood[proposal->world->G], proposal->world->heat);
 	
 	debugtreeout (stdout, crawlback (proposal->world->root->next),
 		      crawlback (proposal->world->root->next), 0);
-      }
+	// }
 #endif
 }
 
@@ -431,7 +433,6 @@ target_oback (proposal_fmt * proposal)
 #ifdef TREEDEBUG
     printf("target_oback\n");
 #endif
-
     copy_x (proposal, proposal->xf, proposal->origin->x);
 #ifdef UEP
 
@@ -443,8 +444,22 @@ target_oback (proposal_fmt * proposal)
             sizeof (MYREAL) * proposal->endsite);
     proposal->v = (proposal->time - proposal->origin->tyme);
     proposal->vs = (proposal->time - proposal->osister->tyme);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->osister->id;
+    bid2 = proposal->osister->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->osister->x, proposal->osister->scale, proposal->vs);
+#endif
     pseudoevaluate (proposal, proposal->xf, proposal->mf, obb, proposal->oback,
                     fabs (proposal->time - obb->tyme));
 }
@@ -478,12 +493,37 @@ target_ocousin (proposal_fmt * proposal)
             sizeof (MYREAL) * proposal->endsite);
     proposal->v = (proposal->time - proposal->origin->tyme);
     proposal->vs = fabs (proposal->ocousin->tyme - proposal->time);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->ocousin->id;
+    bid2 = proposal->ocousin->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->xt, proposal->mt, proposal->vs);
+#endif
     proposal->v = fabs (proposal->time - obb->tyme);
     proposal->vs = obb->tyme - proposal->osister->tyme;
+#ifdef BEAGLE
+    id0  = obb->id;
+    id1  = proposal->oback->id;
+    bid1 = proposal->oback->bid;
+    v1   = proposal->v;
+    id2  = proposal->osister->id;
+    bid2 = proposal->osister->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->osister->x, proposal->osister->scale, proposal->vs);
+#endif
     pseudoevaluate (proposal, proposal->xf, proposal->mf,
                     showtop (crawlback (obb)), obb,
                     (showtop (crawlback (obb))->tyme - obb->tyme));
@@ -510,9 +550,23 @@ target_rbcoa (proposal_fmt * proposal)
 
         memcpy (proposal->mt, proposal->osister->scale,
                 sizeof (MYREAL) * proposal->endsite);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = obb->id;
+    id1  = proposal->osister->id;
+    bid1 = proposal->osister->bid;
+    v1   = proposal->vs;
+    id2  = proposal->ocousin->id;
+    bid2 = proposal->ocousin->bid;
+    v2   = proposal->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs,
                       proposal->ocousin->x, proposal->ocousin->scale,
                       proposal->v);
+#endif
         pseudoevaluate (proposal, proposal->xt, proposal->mt,
                         showtop (crawlback (obb)), obb,
                         showtop (crawlback (obb))->tyme - obb->tyme);
@@ -527,8 +581,19 @@ target_rbcoa (proposal_fmt * proposal)
 
         memcpy (proposal->mf, proposal->origin->scale,
                 sizeof (MYREAL) * proposal->endsite);
+#ifdef BEAGLE
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->target->id;
+    bid2 = proposal->target->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
+#endif
     }
     else
     {
@@ -558,9 +623,23 @@ target_obbcoa (proposal_fmt * proposal)
 
         memcpy (proposal->mt, proposal->osister->scale,
                 sizeof (MYREAL) * proposal->endsite);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->oback->id;
+    id1  = proposal->osister->id;
+    bid1 = proposal->osister->bid;
+    v1   = proposal->vs;
+    id2  = proposal->ocousin->id;
+    bid2 = proposal->ocousin->bid;
+    v2   = proposal->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs,
                       proposal->ocousin->x, proposal->ocousin->scale,
                       proposal->v);
+#endif
         proposal->v = proposal->time - proposal->origin->tyme;
         proposal->vs = fabs (obb->tyme - proposal->time);
         copy_x (proposal, proposal->xf, proposal->origin->x);
@@ -572,8 +651,19 @@ target_obbcoa (proposal_fmt * proposal)
 
         memcpy (proposal->mf, proposal->origin->scale,
                 sizeof (MYREAL) * proposal->endsite);
+#ifdef BEAGLE
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = obb->id;
+    bid2 = obb->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
+#endif
         proposal->v = fabs (showtop (crawlback (obb))->tyme - proposal->time);
         obbb = showtop (crawlback (obb));
         pseudoevaluate (proposal, proposal->xf, proposal->mf, obbb, obb,
@@ -674,9 +764,23 @@ target_stancoa (proposal_fmt * proposal)
 
         memcpy (proposal->mt, proposal->target->scale,
                 sizeof (MYREAL) * proposal->endsite);
-        //CHECK HEREH      memcpy(proposal->mt,proposal->osister->scale, sizeof(MYREAL)*proposal->endsite);
+        //CHECK HERE      memcpy(proposal->mt,proposal->osister->scale, sizeof(MYREAL)*proposal->endsite);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->target->id;
+    bid2 = proposal->target->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
+#endif
         proposal->v =
             fabs (((proposalack =
                         showtop (crawlback (proposal->target)))->tyme -
@@ -686,8 +790,19 @@ target_stancoa (proposal_fmt * proposal)
             children (proposalack, &d1, &d2);
             if (d1 == proposal->target)
                 d1 = d2;
+#ifdef BEAGLE
+	    id0  = proposalack->id;
+	    id1  = proposal->oback->id;
+	    bid1 = proposal->oback->bid;
+	    v1   = proposal->v;
+	    id2  = d1->id;
+	    bid2 = d1->bid;
+	    v2   = d1->v;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
             pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                           d1->x, d1->scale, d1->v);
+#endif
             oldnn = proposalack;
             nn = showtop (crawlback (oldnn));
             while (nn != proposal->oback)
@@ -695,8 +810,19 @@ target_stancoa (proposal_fmt * proposal)
                 children (nn, &d1, &d2);
                 if (d1 == oldnn)
                     d1 = d2;
+#ifdef BEAGLE
+		id0  = nn->id;
+		id1  = oldnn->id;
+		bid1 = oldnn->bid;
+		v1   = oldnn->v;
+		id2  = d1->id;
+		bid2 = d1->bid;
+		v2   = d1->v;
+		prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
                 pseudonuview (proposal, proposal->xf, proposal->mf, oldnn->v,
                               d1->x, d1->scale, d1->v);
+#endif
                 oldnn = nn;
                 nn = showtop (crawlback (nn));
             }
@@ -720,8 +846,22 @@ t_s_upper (proposal_fmt * proposal, node * connect, node * obb)
 
     memcpy (proposal->mf, proposal->origin->scale,
             sizeof (MYREAL) * proposal->endsite);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->target->id;
+    bid2 = proposal->target->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->target->x, proposal->target->scale, proposal->vs);
+#endif
     nn = showtop (crawlback (proposal->target));
     oldnn = proposal->target;
     proposal->vs = fabs (nn->tyme - proposal->time);
@@ -730,8 +870,19 @@ t_s_upper (proposal_fmt * proposal, node * connect, node * obb)
         children (nn, &d1, &d2);
         if (d1 == oldnn)
             d1 = d2;
+#ifdef BEAGLE
+	id0  = nn->id;
+	id1  = oldnn->id;
+	bid1 = oldnn->bid;
+	v1   = proposal->vs;
+	id2  = d1->id;
+	bid2 = d1->bid;
+	v2   = d1->v;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs, d1->x,
                       d1->scale, d1->v);
+#endif
         proposal->vs = nn->v;
         oldnn = nn;
         nn = showtop (crawlback (nn));
@@ -739,18 +890,55 @@ t_s_upper (proposal_fmt * proposal, node * connect, node * obb)
     children (nn, &d1, &d2);
     if (d1 == oldnn)
         d1 = d2;
+#ifdef BEAGLE
+    id0  = nn->id;
+    id1  = oldnn->id;
+    bid1 = oldnn->bid;
+    v1   = proposal->vs;
+    id2  = d1->id;
+    bid2 = d1->bid;
+    v2   = d1->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs, d1->x,
                   d1->scale, d1->v);
+#endif
     proposal->v = obb->tyme - proposal->osister->tyme;
     proposal->vs = obb->tyme - proposal->ocousin->tyme;
     if (connect == proposal->ocousin)
+      {
+#ifdef BEAGLE
+	id0  = obb->id;
+	id1  = proposal->osister->id;
+	bid1 = proposal->osister->bid;
+	v1   = proposal->v;
+	id2  = proposal->ocousin->id;
+	bid2 = proposal->ocousin->bid;
+	v2   = proposal->vs;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->vs,
                       proposal->osister->x, proposal->osister->scale,
                       proposal->v);
+#endif
+      }
     else
+      {
+#ifdef BEAGLE
+	id0  = obb->id;
+	id1  = proposal->osister->id;
+	bid1 = proposal->osister->bid;
+	v1   = proposal->v;
+	id2  = proposal->ocousin->id;
+	bid2 = proposal->ocousin->bid;
+	v2   = proposal->vs;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->ocousin->x, proposal->ocousin->scale,
                       proposal->vs);
+#endif
+      }
     pseudoevaluate (proposal, proposal->xf, proposal->mf,
                     showtop (crawlback (obb)), obb, obb->v);
 }
@@ -761,8 +949,23 @@ t_s_obbncon (proposal_fmt * proposal, node * obb)
 
     node *nn, *oldnn, *d1, *d2;
     /*     FPRINTF(stdout,"t_s_obbncon\n"); */
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    //    printf("????????\n");
+    id0  = obb->id;
+    id1  = proposal->osister->id;
+    bid1 = proposal->osister->bid;
+    v1   = proposal->vs;
+    id2  = proposal->ocousin->id;
+    bid2 = proposal->ocousin->bid;
+    v2   = proposal->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs,
                   proposal->ocousin->x, proposal->ocousin->scale, proposal->v);
+#endif
     nn = showtop (crawlback (obb));
     oldnn = obb;
     proposal->vs = fabs (nn->tyme - obb->tyme);
@@ -771,8 +974,19 @@ t_s_obbncon (proposal_fmt * proposal, node * obb)
         children (nn, &d1, &d2);
         if (d1 == oldnn)
             d1 = d2;
+#ifdef BEAGLE
+	id0  = nn->id;
+	id1  = oldnn->id;
+	bid1 = oldnn->bid;
+	v1   = proposal->vs;
+	id2  = d1->id;
+	bid2 = d1->bid;
+	v2   = d1->v;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs, d1->x,
                       d1->scale, d1->v);
+#endif
         proposal->vs = nn->v;
         oldnn = nn;
         nn = showtop (crawlback (nn));
@@ -784,26 +998,69 @@ void
 t_s_tncon (proposal_fmt * proposal, node * obb)
 {
     node *nn, *oldnn, *d1, *d2;
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->target->id;
+    bid2 = proposal->target->bid;
+    v2   = proposal->time - proposal->target->tyme;
+    // connect oback, origin, target
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                   proposal->target->x, proposal->target->scale,
                   proposal->time - proposal->target->tyme);
+#endif
     nn = showtop (crawlback (proposal->target));
-    oldnn = proposal->target;
+#ifdef BEAGLE
+    oldnn = proposal->oback;
+#else
+    oldnn = proposal->target;    
+#endif
+    // calculates the time between the newly inserted oback and the back of target
     proposal->v = fabs (nn->tyme - proposal->time);
     while (nn != proposal->connect)
-    {
+      {
         children (nn, &d1, &d2);
         if (d1 == oldnn)
-            d1 = d2;
-        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, d1->x,
-                      d1->scale, d1->v);
+	  d1 = d2;
+#ifdef BEAGLE
+	id0  = nn->id;
+	id1  = oldnn->id;
+	bid1 = oldnn->bid;
+	v1   = oldnn->v;
+	id2  = d1->id;
+	bid2 = d2->bid;
+	v2   = d2->v;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
+        pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, d1->x, d1->scale, d1->v);
+#endif
         proposal->v = nn->v;
         oldnn = nn;
         nn = showtop (crawlback (nn));
-    }
+      }
     if (obb != proposal->connect)
+      {
+#ifdef BEAGLE
+	id0  = nn->id;
+	id1  = oldnn->id;
+	bid1 = oldnn->bid;
+	v1   = proposal->v;
+	id2  = obb->id;
+	bid2 = obb->bid;
+	v2   = obb->v;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v,
                       proposal->xt, proposal->mt, proposal->vs);
+#endif
+      }
     proposal->v = proposal->connect->v;
     pseudoevaluate (proposal, proposal->xf, proposal->mf,
                     showtop (crawlback (proposal->connect)), proposal->connect,
@@ -818,9 +1075,34 @@ t_s_tcon (proposal_fmt * proposal, node * uppern)
     if (d1 == uppern)
         d1 = d2;
     proposal->vs = fabs (proposal->target->tyme - showtop (uppern)->tyme);
+#ifdef BEAGLE
+    long id0, id1, id2;
+    long bid1, bid2;
+    double v1, v2;
+    id0  = proposal->target->id;
+    id1  = uppern->id;
+    bid1 = uppern->bid;
+    v1   = proposal->vs;
+    id2  = d1->id;
+    bid2 = d1->bid;
+    v2   = d1->v;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xt, proposal->mt, proposal->vs, d1->x, d1->scale, d1->v); /* eval target */
+#endif
     proposal->vs = proposal->time - proposal->target->tyme;
+#ifdef BEAGLE
+    id0  = proposal->oback->id;
+    id1  = proposal->origin->id;
+    bid1 = proposal->origin->bid;
+    v1   = proposal->v;
+    id2  = proposal->target->id;
+    bid2 = proposal->target->bid;
+    v2   = proposal->vs;
+    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
     pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, proposal->xt, proposal->mt, proposal->vs); /*eval oback */
+#endif
     obb = showtop (crawlback (proposal->target));
     proposal->v = fabs (obb->tyme - proposal->time);
     if (obb->type == 'r')
@@ -830,8 +1112,19 @@ t_s_tcon (proposal_fmt * proposal, node * uppern)
         children (obb, &d1, &d2);
         if (d1 == proposal->target)
             d1 = d2;
+#ifdef BEAGLE
+	id0  = obb->id;
+	id1  = proposal->oback->id;
+	bid1 = proposal->oback->bid;
+	v1   = proposal->v;
+	id2  = d1->id;
+	bid2 = d1->bid;
+	v2   = d1->v;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, proposal->xf, proposal->mf, proposal->v, d1->x,
                       d1->scale, d1->v);
+#endif
         pseudoevaluate (proposal, proposal->xf, proposal->mf,
                         showtop (crawlback (obb)), obb, obb->v);
     }
@@ -1025,9 +1318,9 @@ adjust (node * theNode, MYREAL tyme, long level)
     }
     else if (theNode->type == 't')
     {
-#ifndef TESTINGDATE
-      theNode->tyme = 0.0;
-#endif
+      //#ifndef TESTINGDATE
+      //theNode->tyme = 0.0;
+      //#endif
         theNode->length = lengthof (theNode);
         ltov (theNode);
         return;
@@ -1157,7 +1450,21 @@ pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
         children (mother, &d1, &d2);
         if (d1 == newdaughter)
             d1 = d2;
+#ifdef BEAGLE
+	long id0, id1, id2;
+	long bid1, bid2;
+	double v1, v2;
+	id0  = mother->id; 
+	id1  = proposal->oback->id;
+	bid1 = proposal->oback->bid;
+	v1 = v;
+	id2  = d1->id;
+	bid2 = d1->bid;
+	v2 = d1->v;
+	prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
         pseudonuview (proposal, x, lx, v, d1->x, d1->scale, d1->v);
+#endif
         oldnn = mother;
         nn = showtop (crawlback (mother));
         while (nn->type != 'r')
@@ -1165,7 +1472,18 @@ pseudoevaluate (proposal_fmt * proposal, xarray_fmt x, MYREAL *lx,
             children (nn, &d1, &d2);
             if (d1 == oldnn)
                 d1 = d2;
+#ifdef BEAGLE
+	    id0  = nn->id; 
+	    id1  = d2->id;
+	    bid1 = d2->bid;
+	    v1 = d2->v;
+	    id2  = d1->id;
+	    bid2 = d1->bid;
+	    v2 = d1->v;
+	    prepare_beagle_instances_proposal(proposal, id0, id1, bid1, v1, id2, bid2, v2, proposal->world->beagle);
+#else
             pseudonuview (proposal, x, lx, oldnn->v, d1->x, d1->scale, d1->v);
+#endif
             oldnn = nn;
             nn = showtop (crawlback (nn));
         }
