@@ -333,12 +333,12 @@ void calculate_event_values(duo **eventbins, long *eventbinnum, MYREAL eventinte
       { 
         lastbin=0;
       }
-    if(bin > 10000 || bin < 0) // tooo many bins and it overflows probably 
-                               //I should stop with some bins like 10000
-      {
-        error("bin is smaller than zero or bigger than 10000");
-        return;
-      }
+    //    if(bin > 10000 || bin < 0) // tooo many bins and it overflows probably 
+    //                           //I should stop with some bins like 10000
+    //  {
+    //    warning("Bin (%li) is smaller than zero or bigger than 10000", bin);
+    //    return;
+    //  }
     diff = bin - lastbin;
     val = 1.0;
     if(from == to)
