@@ -16,7 +16,7 @@
  
   Copyright 2002 Peter Beerli and Joseph Felsenstein, Seattle WA
   Copyright 2003 Peter Beerli, Tallahassee FL
-  $Id: migration.h 1453 2008-10-25 01:39:28Z beerli $
+  $Id: migration.h 1564 2009-08-01 00:30:52Z beerli $
   *----------------------------------------------------------------
 */
 /**
@@ -84,6 +84,51 @@ typedef struct _burnin_record_fmt
   long worker;
 } burnin_record_fmt;
 
+
+#ifdef BEAGLE
+typedef struct _beagle_fmt {
+  boolean instance;
+  int *instance_handle; // indicator to the workorder for the GPU/CPU likelihood calculation
+  int *instance_handle1; // 
+  int *instance_handle2; // 
+  int numallocpartials;
+  int numpartials;
+  double *partials; //  conditional likelihood for each site for each parent, leftchild, rightchild
+  int  numallocallyweights; // pattern weighting
+  int *allyweights;// pattern weighting
+  double *weights; //????
+
+  int numallocbranches;
+  int numbranches;
+  double *branch_lengths; // branchlengths for (for each edge leading to a child)
+  int *branch_indices;    // indicator for transition matices
+  int numallocoperations;
+  int numoperations;
+  int *operations;  // holds the operation for each parent,leftchild,lefttransitionmatrix,rightchild,rightt.. 
+
+  int numallocbranches1;
+  int numbranches1;
+  double *branch_lengths1; // branchlengths for (for each edge leading to a child)
+  int *branch_indices1;    // indicator for transition matices
+  int numallocoperations1;
+  int numoperations1;
+  int *operations1;  // holds the operation for each parent,leftchild,lefttransitionmatrix,rightchild,rightt.. 
+
+  int numallocbranches2;
+  int numbranches2;
+  double *branch_lengths2; // branchlengths for (for each edge leading to a child)
+  int *branch_indices2;    // indicator for transition matices
+  int numallocoperations2;
+  int numoperations2;
+  int *operations2;  // holds the operation for each parent,leftchild,lefttransitionmatrix,rightchild,rightt.. 
+
+  int *scalingfactorsindices;
+  int scalingfactorscount;
+
+} beagle_fmt;
+#endif /*BEAGLE*/
+
+
 /// likelihood ratio test data structure
 typedef struct lr_data_fmt
 {
@@ -228,7 +273,7 @@ typedef struct _data
     long **numalleles;      //
   tipdate_fmt ***sampledates;
   MYREAL maxsampledate;
-    char ***indnames;       //!< array of individual names per population
+    char ****indnames;       //!< array of individual names per population
     long numpop;            //!< number of populations
     long loci;              //!< number of loci
   long *position;           //!< position on chromosome [what to do with mutiple chromosomes?]
@@ -237,6 +282,12 @@ typedef struct _data
     MYREAL freqlast;        //
     char dlm;               //!< delimiter for allelic data
     boolean hasghost;       //!< whether there is a population with no data or not \todo check if hasghost is still needed
+  long ***shuffled;         //!< holds the random subset of individuals
+  boolean oneliner;         //!< if a () is used in the sites lines it is assumed that all loci are on one line
+  long *datatype;
+  char *locitypes;          //!< holds all datatypes for each locus, options->datatype is the default
+  char **locusname;
+  long *subloci;            //!< holds the number of strictly linked "loci" that make up the locus for migrate
 }
 data_fmt;
 
@@ -473,7 +524,9 @@ typedef struct _option
     /* mcmc options */
     boolean bayes_infer;
   boolean integrated_like;
+  /* slice sampling*/
   boolean slice_sampling[PRIOR_SIZE];
+  MYREAL *slice_sticksizes;
     MYREAL updateratio;
     int bayesprior[3];
   int bayespretty;
@@ -493,6 +546,9 @@ typedef struct _option
     MYREAL alphavalue;
     MYREAL *mu_rates;
     long muloci;
+  long *newpops;
+  long newpops_numalloc;
+  long newpops_numpop;
   MYREAL *inheritance_scalars;
   long inheritance_scalars_numalloc;
   boolean has_datefile;
@@ -538,10 +594,23 @@ typedef struct _option
   MYREAL *mutationrate_year; // keeps mutation rate per year for each locus 
   long mutationrate_year_numalloc; // 
   MYREAL generation_year;
-
+  long randomsubset;
 }
 option_fmt;
 
+//defines the mutations models used in the extended "Ican do everything mutation model string thing"
+typedef struct _mutationmodel {
+  long numpatterns; // number of site patterns
+  long numsites;
+  long numstates; // number of states in model: DNA=4, DNA+gap=5, msat>2
+  double *basefreqs;
+  long numsiterates;
+  double *sitesrates;
+  char datatype; //specifices the model
+  double *eigenvectormatrix;
+  double *inverseeigenvectormatrix;
+  double *eigenvalues;
+} mutationmodel_fmt;
 
 /* used in the tree structure */
 /// defines a node in the tree structure
@@ -554,6 +623,7 @@ typedef struct _node
     long pop;
     long actualpop;
     long id;
+  long bid;
   boolean visited;
     xarray_fmt x;
 #ifdef UEP
@@ -562,17 +632,14 @@ typedef struct _node
     ueparray_fmt ux;
     MYREAL uepscale;
 #endif
-
     MYREAL *s;
     MYREAL *scale;
     char *nayme;
     boolean top;
     boolean dirty;
-    MYREAL v, tyme;
+  MYREAL v;
+  MYREAL tyme;
     MYREAL length;
-    // MYREAL xcoord;
-    //short ycoord,  ymin, ymax;
-    //MYREAL status;?????
 }
 node;
 
@@ -873,6 +940,7 @@ typedef struct _worldoption
     boolean fastlike;
     boolean bayes_infer;
   boolean slice_sampling[PRIOR_SIZE];
+  MYREAL *slice_sticksizes;
   MYREAL updateratio;
   boolean has_bayesfile;
   boolean has_bayesmdimfile;
@@ -950,14 +1018,16 @@ typedef struct _world
 #ifdef MPI
   int *who;
   int *mpistack;
+  long mpistack_numalloc;
   mpirequest_fmt *mpistack_request;
+  long mpistack_request_numalloc;
   long mpistack_requestnum;
   long mpistacknum;
 #ifdef SLOWNET
   
   int *profilewho;
 #endif
-#endif
+#endif /*MPI*/
   
   char *buffer;   // buffer for profiles, not needed before profiletables()
   long loci;
@@ -1096,7 +1166,18 @@ typedef struct _world
   node **node_collection;
   long node_collection_count;
   long node_collection_allocated;
-
+#ifdef MPI
+#ifdef PARALIO
+  MPI_File mpi_bayesmdimfile;
+#endif
+#endif
+  long *nummutationmodels;//per locus
+  long *sublocistarts;//per locus
+  long *maxnumpattern;//per locus
+  mutationmodel_fmt *mutationmodels;
+#ifdef BEAGLE
+  beagle_fmt *beagle;
+#endif
 }
 world_fmt;
 
@@ -1253,6 +1334,12 @@ typedef  struct _proposal_fmt
     MYREAL *mig0list;
     long *design0list;
     MYREAL treelen;
+
+#ifdef BEAGLE
+  long parentid;
+  long leftid;
+  long rightid;
+#endif
 } proposal_fmt;
 
 
