@@ -16,7 +16,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: reporter.c 1458 2008-11-28 02:38:18Z beerli $
+$Id: reporter.c 1565 2009-08-03 14:48:56Z beerli $
  
 -------------------------------------------------------*/
 /* \file reporter.c 
@@ -384,7 +384,7 @@ void print_bayes_ess(FILE *file, world_fmt *world, long numparam, int offset,
 		  m2mm(pa0,numpop,&frompop,&topop);
 		  if((pa0==pa) && (custm2[pa0]=='*'))
 		    {
-		      pos += sprintf(buffer+pos,"%*.*s%s_(%li,%li)               % -5.5f           %10.2f\n", offset, offset, " ", "M", frompop+1, topop+1,autocorr[pa],effsample[pa]);
+		      pos += sprintf(buffer+pos,"%*.*s%s_%li->%li               % -5.5f           %10.2f\n", offset, offset, " ", "M", frompop+1, topop+1,autocorr[pa],effsample[pa]);
 		    }
 		  else
 		    {
