@@ -16,7 +16,7 @@
  works and are not responsible for any damage you may cause or have.
  
  
- $Id: sequence.c 1448 2008-10-21 03:07:28Z beerli $
+ $Id: sequence.c 1559 2009-06-22 23:57:06Z beerli $
  
 -------------------------------------------------------*/
 /* \file sequence.c
@@ -27,6 +27,7 @@
 #include "tools.h"
 #include "migrate_mpi.h"
 #include "watterson.h"
+#include "tree.h"
 
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
@@ -53,15 +54,15 @@ MYREAL treelike_snp (world_fmt * world, long locus);
 void getbasefreqs (option_fmt * options, seqmodel_fmt * seq, long locus);
 void empiricalfreqs (world_fmt * world, option_fmt * options,
                      seqmodel_fmt * seq, long locus);
-void makeweights (world_fmt * world, data_fmt * data, long locus);
+void makeweights (world_fmt * world, data_fmt * data, option_fmt *options, long locus);
 void makevalues_seq (world_fmt * world, option_fmt * options, data_fmt * data,
                      long locus);
 void make_invarsites (world_fmt * world, data_fmt * data, long locus);
 void make_invarsites_unlinked (world_fmt * world, data_fmt * data,
                                long locus);
-void sitecombine2 (world_fmt * world, data_fmt * data, long sites,
+void sitecombine2 (world_fmt * world, data_fmt * data, option_fmt *options, long sites,
                    long locus);
-void sitesort2 (world_fmt * world, data_fmt * data, long sites, long locus);
+void sitesort2 (world_fmt * world, data_fmt * data, option_fmt *options, long sites, long locus);
 void sitescrunch2 (world_fmt * world, long sites, long i, long j, long locus);
 void inputoptions (world_fmt * world, option_fmt * options, data_fmt * data,
                    long locus);
@@ -133,7 +134,7 @@ init_sequences_aliases (world_fmt * world, option_fmt * options,
     inputoptions (world, options, data, locus);
     if (!options->freqsfrom)
         getbasefreqs (options, world->data->seq[0], locus);
-    makeweights (world, data, locus);
+    makeweights (world, data, options, locus);
 }
 
 
@@ -347,7 +348,17 @@ make_sequences (world_fmt * world, option_fmt * options, data_fmt * data,
     {
       empiricalfreqs (world, options, world->data->seq[0], locus);
       getbasefreqs (options, world->data->seq[0], locus);
-    }
+    } 
+#ifdef BEAGLE
+  if(world->mutationmodels[world->sublocistarts[locus]].basefreqs==NULL)
+    world->mutationmodels[world->sublocistarts[locus]].basefreqs = (double*) calloc(4,sizeof(double));
+  world->mutationmodels[world->sublocistarts[locus]].basefreqs[0] =  world->data->seq[0]->basefrequencies[0];
+  world->mutationmodels[world->sublocistarts[locus]].basefreqs[1] =  world->data->seq[0]->basefrequencies[1];
+  world->mutationmodels[world->sublocistarts[locus]].basefreqs[2] =  world->data->seq[0]->basefrequencies[2];
+  world->mutationmodels[world->sublocistarts[locus]].basefreqs[3] =  world->data->seq[0]->basefrequencies[3];
+  world->mutationmodels[world->sublocistarts[locus]].numstates=4;
+#endif
+  
 }
 
 /// 
@@ -452,7 +463,7 @@ getbasefreqs (option_fmt * options, seqmodel_fmt * seq, long locus)
 /*===================================================*/
 
 void
-makeweights (world_fmt * world, data_fmt * data, long locus)
+makeweights (world_fmt * world, data_fmt * data, option_fmt *options, long locus)
 {
     /* make up weights vector to avoid duplicate computations */
     long i;
@@ -464,8 +475,8 @@ makeweights (world_fmt * world, data_fmt * data, long locus)
         world->data->seq[0]->aliasweight[i] = world->data->seq[0]->weight[i];
         world->data->seq[0]->location[i] = 0;
     }
-    sitesort2 (world, data, world->data->seq[0]->sites[locus], locus);
-    sitecombine2 (world, data, world->data->seq[0]->sites[locus], locus);
+    sitesort2 (world, data, options, world->data->seq[0]->sites[locus], locus);
+    sitecombine2 (world, data, options, world->data->seq[0]->sites[locus], locus);
     sitescrunch2 (world, world->data->seq[0]->sites[locus], 1, 2, locus);
     for (i = 1; i <= world->data->seq[0]->sites[locus]; i++)
     {
@@ -499,154 +510,177 @@ init_sequences2 (world_fmt * world, seqmodel_fmt * seq, long locus)
       }
 }
 
+
+void set_nucleotide(MYREAL *treedata, const char nucleotide, const MYREAL seqerr)
+{
+  long b;
+  const MYREAL seqerr3 = seqerr / 3.;
+  const MYREAL seqerr23 = 2. * seqerr / 3.;
+  const MYREAL oneseqerr = 1.0 - seqerr;
+  const MYREAL oneseqerr23 = 1.0 - seqerr23;
+  const MYREAL oneseqerr3 = 1.0 - seqerr3;
+
+  for (b = 0; b < 4; b++)
+    treedata[b] = 0.0 + seqerr3;
+  switch (nucleotide)
+    {
+    case 'A':
+      treedata[NUC_A] = oneseqerr;
+      break;
+      
+    case 'C':
+      treedata[NUC_C] = oneseqerr;
+      break;
+      
+    case 'G':
+      treedata[NUC_G] = oneseqerr;
+      break;
+      
+    case 'T':
+      treedata[NUC_T] = oneseqerr;
+      break;
+      
+    case 'U':
+      treedata[NUC_T] = oneseqerr;
+      break;
+      
+    case 'M':
+      treedata[NUC_A] = oneseqerr;
+      treedata[NUC_C] = oneseqerr;
+      break;
+      
+    case 'R':
+      treedata[NUC_A] = oneseqerr23;
+      treedata[NUC_G] = oneseqerr23;
+      break;
+      
+    case 'W':
+      treedata[NUC_A] = oneseqerr23;
+      treedata[NUC_T] = oneseqerr23;
+      break;
+      
+    case 'S':
+      treedata[NUC_C] = oneseqerr23;
+      treedata[NUC_G] = oneseqerr23;
+      break;
+      
+    case 'Y':
+      treedata[NUC_C] = oneseqerr23;
+      treedata[NUC_T] = oneseqerr23;
+      break;
+      
+    case 'K':
+      treedata[NUC_G] = oneseqerr23;
+      treedata[NUC_T] = oneseqerr23;
+      break;
+      
+    case 'B':
+      treedata[NUC_A] = seqerr;
+      treedata[NUC_C] = oneseqerr3;
+      treedata[NUC_G] = oneseqerr3;
+      treedata[NUC_T] = oneseqerr3;
+      break;
+      
+    case 'D':
+      treedata[NUC_A] = oneseqerr3;
+      treedata[NUC_C] = seqerr;
+      treedata[NUC_G] = oneseqerr3;
+      treedata[NUC_T] = oneseqerr3;
+      break;
+      
+    case 'H':
+      treedata[NUC_A] = oneseqerr3;
+      treedata[NUC_C] = oneseqerr3;
+      treedata[NUC_G] = seqerr;
+      treedata[NUC_T] = oneseqerr3;
+      break;
+      
+    case 'V':
+      treedata[NUC_A] = oneseqerr3;
+      treedata[NUC_C] = oneseqerr3;
+      treedata[NUC_G] = oneseqerr3;
+      treedata[NUC_T] = seqerr;
+      break;
+      
+    case 'N':
+      for (b = 0; b < 4; b++)
+	treedata[b] = 1.0;
+      break;
+      
+    case 'X':
+      for (b = 0; b < 4; b++)
+	treedata[b] = 1.0;
+      break;
+      
+    case '?':
+      for (b = 0; b < 4; b++)
+	treedata[b] = 1.0;
+      break;
+      
+    case 'O':
+      for (b = 0; b < 4; b++)
+	treedata[b] = 1.0;
+      break;
+#ifdef GAP                            
+    case '-':
+      treedata[4] = 1.0;
+      break;
+#else
+    case '-':
+      for (b = 0; b < 4; b++)
+	treedata[b] = 1.0;
+      break;
+#endif
+    }
+}
+
+
 void
 makevalues_seq (world_fmt * world, option_fmt * options, data_fmt * data,
                 long locus)
 {
-    long i, ii, j, k, l, pop;
-    long b;
-    node **treenode = world->nodep;
-    MYREAL seqerr = options->seqerror;
-    MYREAL seqerr3 = seqerr / 3.;
-    MYREAL seqerr23 = 2. * seqerr / 3.;
-    for (k = 0; k < world->data->seq[0]->endsite; k++)
-    {
-        j = world->data->seq[0]->alias[k] - 1;
-        i = -1;
-        for (pop = 0; pop < world->numpop; pop++)
-        {
-            for (ii = 0; ii < data->numalleles[pop][locus]; ii++)
-            {
-                i++;
-                if (k==0 && !options->usertree)
-                {
-                    strcpy (treenode[i]->nayme, data->indnames[pop][ii]);
-		    treenode[i]->tyme = world->data->sampledates[pop][locus][ii].date 
-		      * world->options->generation_year 
-		      * world->options->mutationrate_year[locus];
-		    world->data->sampledates[pop][locus][ii].id = treenode[i]->id;
+  long ii, ind, j, k, l, pop, top;
+  long z=0;
+  node **treenode = world->nodep;
+  MYREAL seqerr = options->seqerror;
+  long zpop;
+  // we need to recalculate the sumtips
+  world->sumtips = 0;
 
-                }
-                for (l = 0; l < world->options->rcategs; l++)
-                {                    
-                    for (b = 0; b < 4; b++)
-                        treenode[i]->x.s[k][l][b] = 0.0 + seqerr3;
-                    switch (data->yy[pop][ii][locus][0][j])
-                    {
-                        case 'A':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr;
-                            break;
-                            
-                        case 'C':
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr;
-                            break;
-                            
-                        case 'G':
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr;
-                            break;
-                            
-                        case 'T':
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr;
-                            break;
-                            
-                        case 'U':
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr;
-                            break;
-                            
-                        case 'M':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr;
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr;
-                            break;
-                            
-                        case 'R':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr23;
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr23;
-                            break;
-                            
-                        case 'W':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr23;
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr23;
-                            break;
-                            
-                        case 'S':
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr23;
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr23;
-                            break;
-                            
-                        case 'Y':
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr23;
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr23;
-                            break;
-                            
-                        case 'K':
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr23;
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr23;
-                            break;
-                            
-                        case 'B':
-                            treenode[i]->x.s[k][l][NUC_A] = seqerr;
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr3;
-                            break;
-                            
-                        case 'D':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_C] = seqerr;
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr3;
-                            break;
-                            
-                        case 'H':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_G] = seqerr;
-                            treenode[i]->x.s[k][l][NUC_T] = 1.0 - seqerr3;
-                            break;
-                            
-                        case 'V':
-                            treenode[i]->x.s[k][l][NUC_A] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_C] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_G] = 1.0 - seqerr3;
-                            treenode[i]->x.s[k][l][NUC_T] = seqerr;
-                            break;
-                            
-                        case 'N':
-                            for (b = 0; b < 4; b++)
-                                treenode[i]->x.s[k][l][b] = 1.0;
-                            break;
-                            
-                        case 'X':
-                            for (b = 0; b < 4; b++)
-                                treenode[i]->x.s[k][l][b] = 1.0;
-                            break;
-                            
-                        case '?':
-                            for (b = 0; b < 4; b++)
-                                treenode[i]->x.s[k][l][b] = 1.0;
-                            break;
-                            
-                        case 'O':
-                            for (b = 0; b < 4; b++)
-                                treenode[i]->x.s[k][l][b] = 1.0;
-                            break;
-#ifdef GAP                            
-                        case '-':
-			  treenode[i]->x.s[k][l][4] = 1.0;
-			  break;
-#else
-                        case '-':
-                            for (b = 0; b < 4; b++)
-                                treenode[i]->x.s[k][l][b] = 1.0;
-                            break;
-#endif
-                    }
-                }
-            }
-	    //	    for (b = 0; b < 4; b++)
-	    // printf("%f ",treenode[i]->x.s[k][l][b]);
-	    // printf("\n");
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      zpop = 0;
+      if(options->randomsubset>0 && options->randomsubset <  data->numind[pop][locus])
+	{	
+	  top = options->randomsubset;
+	}
+      else
+	{
+	  top = data->numind[pop][locus];
+	}
+      for (ii = 0; ii < top; ii++)
+	{
+	  ind = data->shuffled[pop][locus][ii];
+	  zpop++;
+	  if (!options->usertree)
+	    {
+	      allocate_tip (world, options, &(treenode[z]), pop, locus, z, ind,
+			    data->indnames[pop][ind][locus]);
+	      //treenode[z]->tyme = find_tipdate(treenode[z]->nayme, pop,world);
+	      world->data->sampledates[pop][locus][ind].id = treenode[z]->id;
+	      z++;
+	    }	    
+	  for (k = 0; k < world->data->seq[0]->endsite; k++)
+	    {
+	      j = world->data->seq[0]->alias[k] - 1;
+	      for (l = 0; l < world->options->rcategs; l++)
+		{                    
+		  set_nucleotide(treenode[z-1]->x.s[k][l],data->yy[pop][ind][locus][0][j], seqerr);
+		}
+	    }
         }
+      data->numalleles[pop][locus] = zpop;
+      world->sumtips += zpop;
     }
 }
 
@@ -656,57 +690,59 @@ void
 makevalues_snp (world_fmt * world, option_fmt * options, data_fmt * data,
                 long locus)
 {
-    long i, ii, j, k, l, pop;
-    long b, f;
-    
-    node **treenode = world->nodep;
-    f = world->data->seq[0]->addon;
-    for (k = 0; k < world->data->seq[0]->endsite * (f + 1);
-            k += (1 + world->data->seq[0]->addon))
+  long i, ii, j, k, l, pop;
+  long b, f;
+  long ind;
+  node **treenode = world->nodep;
+  f = world->data->seq[0]->addon;
+  for (k = 0; k < world->data->seq[0]->endsite * (f + 1);
+       k += (1 + world->data->seq[0]->addon))
     {
-        j = world->data->seq[0]->alias[k / (f + 1)] - 1;
-        i = -1;
-        for (pop = 0; pop < world->numpop; pop++)
-        {
-            for (ii = 0; ii < data->numalleles[pop][locus]; ii++)
-            {
-                i++;
-                if (k==0 && !options->usertree)
-                {
-                    strcpy (treenode[i]->nayme, data->indnames[pop][ii]);
-                }
-                for (l = 0; l < world->options->rcategs; l++)
-                {
-                    if (pop == 0)
-                    {
+      j = world->data->seq[0]->alias[k / (f + 1)] - 1;
+      i = -1;
+      for (pop = 0; pop < data->numpop; pop++)
+	{
+	  for (ii = 0; ii < data->numalleles[pop][locus]; ii++)
+	    {
+	      ind = data->shuffled[pop][locus][ii];
+	      i++;
+	      if (k==0 && !options->usertree)
+		{
+		  strcpy (treenode[i]->nayme, data->indnames[pop][ind][locus]);
+		  treenode[i]->tyme = find_tipdate(treenode[i]->nayme, pop,world);
+		}
+	      for (l = 0; l < world->options->rcategs; l++)
+		{
+		  if (pop == 0)
+		    {
 		      //                        treenode[i]->x.s[k][l] = vec_float_one();
-                        // reset all values for the snp columns
-                      //  treenode[i]->x.s[k][l] = vec_zero();
-                        for (b = 1; b <= 4; b++)
-                        {
-                            memset (treenode[i]->x.s[k + b][l], 0,
-                                    sizeof (MYREAL) * 4);
-                            treenode[i]->x.s[k + b][l][b - 1] = 1.0;
-                        }
-                        continue;
-                    }
-                    else
-                    {
+		      // reset all values for the snp columns
+		      //  treenode[i]->x.s[k][l] = vec_zero();
+		      for (b = 1; b <= 4; b++)
+			{
+			  memset (treenode[i]->x.s[k + b][l], 0,
+				  sizeof (MYREAL) * 4);
+			  treenode[i]->x.s[k + b][l][b - 1] = 1.0;
+			}
+		      continue;
+		    }
+		  else
+		      {
                         for (b = 0; b <= 4; b++)
-                            memset (treenode[i]->x.s[k + b][l], 0,
-                                    sizeof (MYREAL) * 4);
-                    }
-
-                    switch (data->yy[pop][ii][locus][0][j])
-                    {
-                    case 'A':
+			  memset (treenode[i]->x.s[k + b][l], 0,
+				  sizeof (MYREAL) * 4);
+		      }
+		    
+                    switch (data->yy[pop][ind][locus][0][j])
+		      {
+		      case 'A':
                         treenode[i]->x.s[k][l][NUC_A] = 1.0;
                         treenode[i]->x.s[k + 1][l][NUC_A] = 1.0;
                         treenode[i]->x.s[k + 2][l][NUC_A] = 1.0;
                         treenode[i]->x.s[k + 3][l][NUC_A] = 1.0;
                         treenode[i]->x.s[k + 4][l][NUC_A] = 1.0;
                         break;
-
+			
                     case 'C':
                         treenode[i]->x.s[k][l][NUC_C] = 1.0;
                         treenode[i]->x.s[k + 1][l][NUC_C] = 1.0;
@@ -913,7 +949,7 @@ make_invarsites (world_fmt * world, data_fmt * data, long locus)
     long endsite = world->data->seq[0]->endsite;
     node **treenode = world->nodep;
     i = -1;
-    for (pop = 0; pop < world->numpop; pop++)
+    for (pop = 0; pop < data->numpop; pop++)
     {
         for (ii = 0; ii < data->numalleles[pop][locus]; ii++)
         {
@@ -939,7 +975,7 @@ make_invarsites_unlinked (world_fmt * world, data_fmt * data, long locus)
     long endsite = world->data->seq[0]->endsite;
     node **treenode = world->nodep;
     i = -1;
-    for (pop = 0; pop < world->numpop; pop++)
+    for (pop = 0; pop < data->numpop; pop++)
     {
         for (ii = 0; ii < data->numalleles[pop][locus]; ii++)
         {
@@ -987,24 +1023,33 @@ make_invarsites_unlinked (world_fmt * world, data_fmt * data, long locus)
 }
 
 void
-sitesort2 (world_fmt * world, data_fmt * data, long sites, long locus)
+sitesort2 (world_fmt * world, data_fmt * data, option_fmt *options, long sites, long locus)
 {
-    long gap, i, i1, j, jj, jg, k, kk, itemp, pop, z = 0;
+  long gap, i, i1, j, jj, jg, k, kk, kkk, itemp, pop, z = 0;
     boolean flip, tied, samewt;
     seqmodel_fmt *seq;
     long *tempsum, *temppop;
-	MYREAL a1,a2;
-    tempsum = (long *) mycalloc (1, sizeof (long) * world->numpop);
-    temppop = (long *) mycalloc (1, sizeof (long) * world->numpop);
-    for (i = 0; i < world->numpop; i++)
+    long numind;
+    MYREAL a1,a2;
+    tempsum = (long *) mycalloc (1, sizeof (long) * data->numpop);
+    temppop = (long *) mycalloc (1, sizeof (long) * data->numpop);
+    for (i = 0; i < data->numpop; i++)
     {
-        if (data->numind[i][locus] > 0)
+      if(options->randomsubset > 0)
+	{
+	  numind = (options->randomsubset < data->numind[i][locus] ? options->randomsubset : data->numind[i][locus]);
+	}
+      else
+	{
+	  numind = data->numind[i][locus] ;
+	}
+      if (numind > 0)
         {
-            temppop[z] = i;
-            if (z == 0)
-                tempsum[z] = data->numind[i][locus];
-            else
-                tempsum[z] = tempsum[z - 1] + data->numind[i][locus];
+	  temppop[z] = i;
+	  if (z == 0)
+	    tempsum[z] = numind;
+	  else
+	    tempsum[z] = tempsum[z - 1] + numind;
             z++;
         }
     }
@@ -1042,9 +1087,10 @@ sitesort2 (world_fmt * world, data_fmt * data, long sites, long locus)
                     {
                         kk++;
                     }
-					i1 = temppop[pop];
-					a1 = data->yy[i1][kk][locus][0][jj - 1];
-					a2 = data->yy[i1][kk][locus][0][jg - 1];
+		    i1 = temppop[pop];
+		    kkk = data->shuffled[i1][locus][kk];
+		    a1 = data->yy[i1][kkk][locus][0][jj - 1];
+		    a2 = data->yy[i1][kkk][locus][0][jg - 1];
                     flip =
                         (a1 > a2);
                     tied = (tied
@@ -1070,24 +1116,35 @@ sitesort2 (world_fmt * world, data_fmt * data, long sites, long locus)
 
 
 void
-sitecombine2 (world_fmt * world, data_fmt * data, long sites, long locus)
+sitecombine2 (world_fmt * world, data_fmt * data, option_fmt *options, long sites, long locus)
 {
     long i, j, k, kk, pop, z = 0;
     boolean tied, samewt;
     seqmodel_fmt *seq;
     long *tempsum, *temppop;
-    tempsum = (long *) mycalloc (1, sizeof (long) * world->numpop);
-    temppop = (long *) mycalloc (1, sizeof (long) * world->numpop);
-    tempsum[0] = data->numind[0][locus];
-    for (i = 0; i < world->numpop; i++)
+    long numind;
+    long kkk;
+    tempsum = (long *) mycalloc (1, sizeof (long) * data->numpop);
+    temppop = (long *) mycalloc (1, sizeof (long) * data->numpop);
+    if(options->randomsubset > 0)
+      {
+	tempsum[0] = (options->randomsubset < data->numind[0][locus] ? options->randomsubset : data->numind[0][locus]);
+      }
+    else
+      {
+	tempsum[0] = data->numind[0][locus];
+      }
+    
+    for (i = 0; i < data->numpop; i++)
     {
-        if (data->numind[i][locus] > 0)
+      numind = ((options->randomsubset > 0) && (options->randomsubset < data->numind[0][locus])) ? options->randomsubset : data->numind[i][locus];
+      if (numind > 0)
         {
-            temppop[z] = i;
-            if (z == 0)
-                tempsum[z] = data->numind[i][locus];
-            else
-                tempsum[z] = tempsum[z - 1] + data->numind[i][locus];
+	  temppop[z] = i;
+	  if (z == 0)
+	    tempsum[z] = numind;
+	  else
+                tempsum[z] = tempsum[z - 1] + numind;
             z++;
         }
     }
@@ -1121,11 +1178,12 @@ sitecombine2 (world_fmt * world, data_fmt * data, long sites, long locus)
                 {
                     kk++;
                 }
+		kkk = data->shuffled[pop][locus][kk];
                 tied = (tied
-                        && data->yy[temppop[pop]][kk][locus][0][seq->
+                        && data->yy[temppop[pop]][kkk][locus][0][seq->
                                                                 alias[i - 1] -
                                                                 1] ==
-                        data->yy[temppop[pop]][kk][locus][0][seq->alias[j - 1] -
+                        data->yy[temppop[pop]][kkk][locus][0][seq->alias[j - 1] -
                                                              1]);
                 k++;
             }
