@@ -56,7 +56,8 @@ MYREAL waiting_theta(long pop,
   
   for(i=0;i<numpop; i++)
     {
-      expected += (lineages[i] * (lineages[i]-1) * (1.0/ param[i]) +  mig0list[i] * lineages[i]) ;
+      if(lineages[i]>0)
+	expected += (lineages[i] * (lineages[i]-1) * (1.0/ param[i]) +  mig0list[i] * lineages[i]) ;
     }
   kpop = lineages[pop] * ( lineages[pop] -1);
   expected = 1./interval - (expected - kpop * 1.0 / param[pop])*inv_mu_rate;
@@ -80,14 +81,16 @@ MYREAL waiting_M(long pop,long to,
   long i;
   long kpop;
   MYREAL expected = 0.0;
-  
+  MYREAL line;
   for(i=0;i<numpop; i++)
     {
-      expected += (lineages[i] * (lineages[i]-1) * (1.0/ param[i]) +  mig0list[i] * lineages[i]);
+      line = lineages[i];
+      if(line>0)
+	expected += (lineages[i] * (lineages[i]-1) * (1.0/ param[i]) +  mig0list[i] * lineages[i]);
     }
   kpop = lineages[to];
   expected = 1./interval - (expected - kpop * param[pop]) * inv_mu_rate ;
-  if(expected > EPSILON)
+  if(kpop > EPSILON)
     return expected / kpop;
   else
     return -9.;
@@ -245,7 +248,8 @@ void calculate_expected_values(tetra **eventbins,
   lweight = 1.0;
   // do we need to report problems when bin number is above 10000?
   // 
-  if(age > 1000.0 || bin > 10000)
+  //  if(age > 1000.0 || bin > 10000)
+  if(age > 1000.0)
     return;
 
   if(firstbin < 0) 
@@ -258,9 +262,10 @@ void calculate_expected_values(tetra **eventbins,
   if(from == to)
     {
       diff = bin - firstbin;
+      weight1 = interval * inv_eventinterval; // or =lweight=1.0
       pop  = to;
       val = waiting_theta(pop,world->param0,world->mig0list, lineages,interval, inv_mu_rate, numpop); 
-      if(val < 0.0)
+      if(val <= 0.0)
 	return;
       //      printf("%i> k(k-1)=%li val=%f pop=%li\n",myID,k,val, pop);
       //se1  = val * val;
@@ -269,13 +274,15 @@ void calculate_expected_values(tetra **eventbins,
   else
     {
       diff = 0;
+      weight1 = 1.0;
       pop = mm2m(from, to, numpop);
 
       val = waiting_M(pop, to, world->param0,world->mig0list, lineages,interval, inv_mu_rate, numpop); 
 
 
       if(val < 0.0)
-	return;
+	val = 0.0;
+      //DEBUG TEST	return;
       //se1 = val * val;
       //se2 = 1.0;
     }
@@ -300,24 +307,27 @@ void calculate_expected_values(tetra **eventbins,
     {
       //start and stop bin are the same the whole interval does not extend over bin boundaries
     case 0: 
-      weight1 = interval * inv_eventinterval; // or =lweight=1.0
-      calc_bin_average(&bins[bin], val, weight1);
+      // weight1 is calculated earlier to make sure that migration weights are 1.0 and the
+      // coalescence weights are the standard weight1
+      calc_bin_average(&bins[firstbin], val, weight1);
       break;
     case 1: 
       // the interval crosses a single bin boundary
       // older bin
-      weight1 = (age - binage) * inv_eventinterval; 
-      calc_bin_average(&bins[bin], val, weight1);
+      //      weight1 = (age - binage) * inv_eventinterval; 
+      weight1 = (-age + interval + firstbinage + eventinterval) * inv_eventinterval;
+      calc_bin_average(&bins[firstbin], val, weight1);
       // newer bin
+      weight2 = (age - binage) * inv_eventinterval;
       calc_bin_average(&bins[bin], val, weight2);
       break;
     default:
       // start and stop are in different bins and more than one boundary apart
       weight1 = (-age + interval + firstbinage + eventinterval) * inv_eventinterval;
-      calc_bin_average(&bins[bin], val, weight1);
+      calc_bin_average(&bins[firstbin], val, weight1);
       for(i=firstbin+1; i < bin; i++)
 	{
-	  calc_bin_average(&bins[bin], val, 1.0);
+	  calc_bin_average(&bins[i], val, 1.0);
 	}
       weight2 = (age - binage) * inv_eventinterval;
     calc_bin_average(&bins[bin], val, weight2);
@@ -331,6 +341,16 @@ void calculate_expected_values(tetra **eventbins,
 ///
 /// set up the skyline plot histogram containers, only when also the migration histograms are recorded
 /// this function needs to be called AFTER the setup_mighist() function
+/// the main vehicle is the evenbins that contain 9 basic bins for each time bin, each parameter
+/// 0) Time
+/// 1) Parameter value (average)
+/// 2) standard deviation  (average standard deviation)
+/// 3) standard deviation of weights (average weight)
+/// 4) Number of values put into bin (average count)
+/// 5) autocorrelation (average autocorrelation coeff)
+/// 6)
+/// 7)
+/// 8)
 void
 setup_expected_events (world_fmt * world, option_fmt * options)
 {
@@ -577,10 +597,7 @@ void prepare_expected_values(world_fmt *world)
 	  for(pop = 0; pop < numpop2; pop++)
 	    {  
 	      sum = (MYREAL) 0.0;
-	      
-	      if(eventbinnum[pop] > eventbinnum_allmax)
-		eventbinnum_allmax = eventbinnum[pop];
-	      
+	      	      
 	      for(i = 0; i < eventbinnum[pop]; i++)
 		{
 		  if(eventbins[pop][i][1] <= 0.0)
@@ -597,6 +614,7 @@ void prepare_expected_values(world_fmt *world)
 		    }
 		  sum += eventbins[pop][i][1];
 		}
+
 	      for(i = 0; i < eventbinnum[pop]; i++)
 		{
 		  if(eventbins[pop][i][1] <= 0.0)
@@ -604,9 +622,16 @@ void prepare_expected_values(world_fmt *world)
 		  // calculate standard deviations of values (using weighted formula)
 		  // and adjusting for correlated samples
 		  eventbins[pop][i][5] /= eventbins[pop][i][2];//calculation of autocorrelation coefficient
-		  rs = eventbins[pop][i][5];
-		  rs = eventbins[pop][i][4] * (1.-rs)/(1.+rs); 
+		  rs = eventbins[pop][i][5];//autocorrelation
+		  rs = eventbins[pop][i][4] * (1.-rs)/(1.+rs);// artificial value correcting for correlation 
+		  // rs is the number of observation in bin
+		  // standard deviation corrected with autocorrelation
+		  // rs is number of obersvations the formula below
+		  // assumes that eventbins[][][1] is the average weight and not the sumweight
+		  // because originally to formula calls for *= rs/((rs-1) sumweight)
+		  // here we do rs/((rs-1) rs * weightaverage)
 		  eventbins[pop][i][2] *= 1./((rs-1.) * eventbins[pop][i][1]);
+
 		  if (MYISNAN (eventbins[pop][i][5]))
 		    {
 		      eventbins[pop][i][5] = -999.;
@@ -618,14 +643,25 @@ void prepare_expected_values(world_fmt *world)
 		    }
 		  else
 		    {
-		      eventbins[pop][i][2] = sqrt(eventbins[pop][i][2]);
+		      eventbins[pop][i][2] = sqrt(eventbins[pop][i][2])/sqrt(eventbins[pop][i][4]);
 		    }
 		  // calculate standard deviations of weights (this were unweighted)
+		  // this seems only adjust for sample versus population but this was done before see up
 		  eventbins[pop][i][3] *= eventbins[pop][i][4]/(eventbins[pop][i][4]-1.); 
 		  eventbins[pop][i][3] = sqrt(eventbins[pop][i][3]);
 		  eventbins[pop][i][1] /= sum; // calculate frequency: sum(weight_i)/sum(sum(weight_i)_j)
 		}
-	    } 
+	      // cutting off the tail of the skyline to avoid too many zeroes.
+	      i = eventbinnum[pop]-1;
+	      while(i>0 && eventbins[pop][i][2]<=0.0)
+		    {
+		      i--;
+		    }
+	      eventbinnum[pop] = i;
+	      // calculating the highest bin number
+	      if(eventbinnum[pop] > eventbinnum_allmax)
+		eventbinnum_allmax = eventbinnum[pop];
+	    }
 	}
     }
   if(world->loci>1)
