@@ -124,8 +124,10 @@ MYREAL slice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (
   MYREAL fzl;
   MYREAL fzr;
   MYREAL priorratio;
-  MYREAL stick = (which < world->numpop) ? 1.0 : 2.0 ;
-  long w;
+  register MYREAL stick = world->options->slice_sticksizes[which];
+  const MYREAL stick_expand = 1.02;
+  const MYREAL stick_reduce = 1.0/1.02;
+  register long w;
   long count;
   // find parameter values to honor the custm-migration matrix
   if(which < world->numpop2)
@@ -177,6 +179,7 @@ MYREAL slice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (
   fzr= probg_treetimes(world) + priorratio;
   while(y < fzl && y > -DBL_MAX)
     {
+      stick *= stick_expand;
       zl -= stick;
       ezl = EXP(zl);
       priorratio = func(ezl, -1. , bayes, w);
@@ -185,6 +188,7 @@ MYREAL slice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (
     }
   while(y < fzr && y > -DBL_MAX)
     {
+      stick *= stick_expand;
       zr += stick;
       ezr = EXP(zr);
       priorratio = func(ezr, -1. ,bayes, w);
@@ -201,6 +205,7 @@ MYREAL slice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (
   count = 0;
   while(fx < y && ((zr-zl) > EPSILON))
     {
+      stick *= stick_reduce;
       count++;
       //      if((x - zl) < (zr - x))
       if(x < log_newstartval)
@@ -219,6 +224,8 @@ MYREAL slice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (
     }
   //fprintf(stdout,"{{%f,%f},{%f,%f},{%f,%f}} (* y=%f, c=%li, %li, lv=%f *)\n",zl,fzl,x,fx,zr,fzr,y, count, which,startval);
   *startval = EXP(x);
+  world->options->slice_sticksizes[which] = stick;
+  //fprintf(stdout,"### %li %f\n",which, stick);
   return fx - priorratio;
 }
 
@@ -227,25 +234,27 @@ MYREAL slice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (
 /// returns the log(probability) and changes startval]
 MYREAL expslice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func) (MYREAL, MYREAL, bayes_fmt *, long))
 {
-  long w;
+  register long w;
   bayes_fmt * bayes = world-> bayes;
-  MYREAL *minparam = bayes->minparam;
-  MYREAL *maxparam = bayes->maxparam;
+  const MYREAL *minparam = bayes->minparam;
+  const MYREAL *maxparam = bayes->maxparam;
   MYREAL newstartval = *startval;
   //  MYREAL log_newstartval;
-  MYREAL ra;
-  MYREAL x;
-  MYREAL fx;
-  MYREAL y;
-  MYREAL r;
-  MYREAL zl;
-  MYREAL zr;
-  MYREAL fzl;
-  MYREAL fzr;
+  register MYREAL ra;
+  register MYREAL x;
+  register MYREAL fx;
+  register MYREAL y;
+  register MYREAL r;
+  register MYREAL zl;
+  register MYREAL zr;
+  register MYREAL fzl;
+  register MYREAL fzr;
   MYREAL priorratio;
-  MYREAL stick;
   long count = 3L;
-  stick = (maxparam[which] - minparam[which]) / 20. ;
+  register MYREAL   stick = world->options->slice_sticksizes[which];
+  const MYREAL stick_expand = 1.02;
+  const MYREAL stick_reduce = 1.0/1.02;
+
 
 #ifdef SLICEREPORTER
   fprintf(stdout,"# start slice --------------------\n");
@@ -313,6 +322,7 @@ MYREAL expslice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func
 #endif
   while(y < fzl)
     {
+      stick *= stick_expand;
       zl -= stick;
       //      ezl = EXP(zl);
       priorratio = func(zl, -1. , bayes, w);
@@ -325,6 +335,7 @@ MYREAL expslice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func
     }
   while(y < fzr)
     {
+      stick *= stick_expand;
       zr += stick;
       //      ezr = EXP(zr);
       priorratio = func(zr, -1. ,bayes, w);
@@ -348,6 +359,7 @@ MYREAL expslice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func
 #endif
   while(fx < y && ((zr-zl) > EPSILON))
     {
+      stick *= stick_reduce;
       count++;
       //      if((x - zl) < (zr - x))
       if(x < newstartval)
@@ -381,6 +393,7 @@ MYREAL expslice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func
   if((x < maxparam[w]) && (x > minparam[w]))
     {
       *startval = x;
+      world->options->slice_sticksizes[which] = stick;
       return fx - priorratio;
     }
   else
@@ -388,11 +401,13 @@ MYREAL expslice (MYREAL *startval, long which, world_fmt * world, MYREAL  (*func
       if(x < minparam[w])
 	{
 	  *startval = minparam[w];
+	  world->options->slice_sticksizes[which] = stick;
 	  return fx - priorratio;
 	}
       else
 	{
 	  *startval = maxparam[w];
+	  world->options->slice_sticksizes[which] = stick;
 	  return fx - priorratio;
 	}
     }
