@@ -20,7 +20,7 @@ Copyright 2003-2006 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: tools.c 1458 2008-11-28 02:38:18Z beerli $
+$Id: tools.c 1563 2009-07-30 18:28:33Z beerli $
 -------------------------------------------------------*/
 /* \file tools.c */
 
@@ -109,7 +109,7 @@ void translate (char *text, char from, char to);
 long count_words (char *text);
 void print_line (FILE * outfile, char c, long nn, long flag);
 void sprint_line (char *buffer, char c, long nn, long flag);
-void add_to_buffer(char *fp, long *bufsize, char **buffer);
+void add_to_buffer(char *fp, long *bufsize, char **buffer, long *allocbufsize);
 
 /* time reporting */
 void get_time (char *nowstr, char ts[]);
@@ -457,7 +457,9 @@ void stop_node_collection(world_fmt *world)
 
 void collect_nodelet(world_fmt *world, node *p)
 {
-  //long i;
+#ifdef MESS
+  long i;
+#endif
   long oldalloc;
 
   if(world->node_collection_count < world->node_collection_allocated)
@@ -489,9 +491,9 @@ void collect_nodelet(world_fmt *world, node *p)
       warning("in node collection phase we should not need to allocate new space\n");
 #endif
       oldalloc = world->node_collection_allocated;
-      world->node_collection_allocated += 400;
+      world->node_collection_allocated += 300;
       world->node_collection = (node **) realloc(world->node_collection, world->node_collection_allocated * sizeof(node*));
-      memset(world->node_collection+oldalloc,0, 400 * sizeof(node*));
+      memset(world->node_collection+oldalloc,0, 300 * sizeof(node*));
       world->node_collection[world->node_collection_count] = p;
 #ifdef MESS
       for(i=0;i<check_collection_count;i++)
@@ -518,21 +520,13 @@ void collect_nodelet(world_fmt *world, node *p)
   //printf("\n");
 }
 
+///
+/// dispense memory for nodes from a list of avialable nodes, allocates more memory 
+/// no nodes are available in the list if nodes
 node *  dispense_nodelet(world_fmt *world)
 {
-  static long oldalloc ;
-  static boolean set=FALSE;
-  //static char * treespace;
-  //long allocval;
-  //long pos;
-  
   long i;
   node *p = NULL;
-  if(!set)
-    {
-      oldalloc= world->node_collection_allocated;
-      set = TRUE;
-    }
   if(world->node_collection_count > 0)
     {
       world->node_collection_count--;
@@ -544,25 +538,32 @@ node *  dispense_nodelet(world_fmt *world)
     }
   else
     {
-      oldalloc += 300;
+      //all node memory is checked out, we
+      //increase available nodes in chunks of 300
+      world->node_collection_allocated += 300;
+      world->node_collection = (node **) myrealloc(world->node_collection,sizeof(node *)* world->node_collection_allocated);
+      // set new slots to NULL
+      memset(world->node_collection + world->node_collection_allocated - 300, 0,sizeof(node *)*300);
+      // we allocate memory of 300 nodes and start at the 0 element because all
+      // elements allocated earlier are checked out and the node_collection array
+      // should contain only NULLs
       for(i = 0; i < 300; i++)
 	{
-	  world->node_collection[i] = (node *) calloc(1, sizeof(node));
+	  world->node_collection[i] = (node *) mycalloc(1, sizeof(node));
 #ifdef MESS
 	  world->node_collection[i]->id = unique_id_global++;
 #endif
 	}
 #ifdef MESS
-      check_collection = (long *) realloc(check_collection, oldalloc * sizeof(long));
+      check_collection = (long *) realloc(check_collection, world->node_collection_allocated * sizeof(long));
 #endif
       world->node_collection_count += 300;
       world->node_collection_count--;
-      p = world->node_collection[world->node_collection_count];
+      p = world->node_collection[world->node_collection_count]; // dispense
+      world->node_collection[world->node_collection_count] = NULL; // remove from list
 #ifdef MESS
       check_collection[check_collection_count++] = p->id;
 #endif
-      world->node_collection[world->node_collection_count] = NULL;
-
     }
   return p;
 }
@@ -674,112 +675,132 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 			setup_filehandle_db((void *) options->logfile, world, options, data);
 #endif			
 		  }
-        
+		
 		if (options->mixplot)
-        {
-			openfile (&options->mixfile, options->mixfilename, "w+",  NULL);
+		  {
+		    openfile (&options->mixfile, options->mixfilename, "w+",  NULL);
 #ifdef MPI
-			setup_filehandle_db((void *) options->mixfile, world, options, data);
+		    setup_filehandle_db((void *) options->mixfile, world, options, data);
 #endif			
-        }
+		  }
 		if (options->aic)
 		  {
-			openfile (&options->aicfile, options->aicfilename, "w+",  NULL);
+		    openfile (&options->aicfile, options->aicfilename, "w+",  NULL);
 #ifdef MPI
-			setup_filehandle_db((void *) options->aicfile, world, options, data);
+		    setup_filehandle_db((void *) options->aicfile, world, options, data);
 #endif
 		  }			
-        if (options->plot)
-        {
-            switch (options->plotmethod)
-            {
-            case PLOTALL:
-                openfile (&world->mathfile, options->mathfilename, "w+", 
-                          NULL);
+		if (options->plot)
+		  {
+		    switch (options->plotmethod)
+		      {
+		      case PLOTALL:
+			openfile (&world->mathfile, options->mathfilename, "w+", 
+				  NULL);
 #ifdef MPI
-                setup_filehandle_db((void *) world->mathfile, world, options, data);
+			setup_filehandle_db((void *) world->mathfile, world, options, data);
 #endif
-                break;
-            default:  /*e.g. 0 this create just the plots in outfile */
-                break;
-            }
-        }
-        if (options->geo)
-	  {
-            openfile (&data->geofile, options->geofilename, "r+",  NULL);
-	  }
-        if (options->has_datefile)
-	  {
-            openfile (&data->datefile, options->datefilename, "r+",  NULL);
-	  }
+			break;
+		      default:  /*e.g. 0 this create just the plots in outfile */
+			break;
+		      }
+		  }
+		if (options->geo)
+		  {
+		    openfile (&data->geofile, options->geofilename, "r+",  NULL);
+		  }
+		if (options->has_datefile)
+		  {
+		    openfile (&data->datefile, options->datefilename, "r+",  NULL);
+		  }
 #ifdef UEP
-
-        if (options->uep)
-            openfile (&data->uepfile, options->uepfilename, "r+",  NULL);
+		
+		if (options->uep)
+		  openfile (&data->uepfile, options->uepfilename, "r+",  NULL);
 #endif
-        if (options->bayes_infer)
-	  {
-	    if(options->has_bayesfile)
-	      openfile (&world->bayesfile, options->bayesfilename, "w+",  NULL);
-	    if(options->has_bayesmdimfile)
-	      {
-		if(options->datatype != 'g')
-		  openfile (&world->bayesmdimfile, options->bayesmdimfilename, "w+",  NULL);
-		else
-		  openfile(&world->bayesmdimfile, options->bayesmdimfilename,"r+",NULL);
-	      }
+		if (options->bayes_infer)
+		  {
+		    if(options->has_bayesfile)
+		      openfile (&world->bayesfile, options->bayesfilename, "w+",  NULL);
+
+		    if(options->has_bayesmdimfile)
+		      {
+			if(options->datatype != 'g')
+			  {
 #ifdef MPI
-	    if(options->has_bayesfile)
-	      setup_filehandle_db((void *) world->bayesfile, world, options,data);
-	    if(options->has_bayesmdimfile)
-	      setup_filehandle_db((void *) world->bayesmdimfile, world, options,data);
+#ifndef PARALIO
+			    openfile (&world->bayesmdimfile, options->bayesmdimfilename, "w+",  NULL);
+#endif			  
+#else
+			    openfile (&world->bayesmdimfile, options->bayesmdimfilename, "w+",  NULL);
+#endif
+			  }
+			else
+			  openfile(&world->bayesmdimfile, options->bayesmdimfilename,"r+",NULL);
+		      }
+#ifdef MPI
+		    if(options->has_bayesfile)
+		      setup_filehandle_db((void *) world->bayesfile, world, options,data);
+		    if(options->has_bayesmdimfile)
+		      {
+#ifdef PARALIO
+			MPI_File_open(comm_world,options->bayesmdimfilename,MPI_MODE_CREATE | MPI_MODE_WRONLY ,MPI_INFO_NULL, &world->mpi_bayesmdimfile);
+#else
+			setup_filehandle_db((void *) world->bayesmdimfile, world, options,data);
+#endif
+		      }
 #endif
 		  }			
 #ifdef MPI
 		tempdb[0]=filenum;
 		for(i=0, j=1;i<filenum;i++)
-		{
-			tempdb[j++] = (long) filedb[i].file;
-			tempdb[j++] = (long) filedb[i].handle;
-		}
+		  {
+		    tempdb[j++] = (long) filedb[i].file;
+		    tempdb[j++] = (long) filedb[i].handle;
+		  }
 		MYMPIBCAST (tempdb, maxfilenum, MPI_LONG, MASTER, comm_world);
 #endif
     }
-	else
-	  {
-		// all non myID==MASTER
+    else
+      {
+	// all non myID==MASTER
 #ifdef MPI
-		MYMPIBCAST (tempdb, maxfilenum, MPI_LONG, MASTER, comm_world);
-		filenum = tempdb[0]; 
-		for(i=0, j=1;i<filenum;i++)
-		  {
-			filedb[i].file = (FILE *) tempdb[j++];
-			filedb[i].handle =  tempdb[j++];
-		  }
-		//assumess same order as in the master
-		j=1; //zero is stdout
-		world->outfile = filedb[j++].file;
-		if (options->treeprint > 0 && (!options->readsum))
-		  world->treefile = filedb[j++].file;
-		if (options->mighist)
-			world->mighistfile = filedb[j++].file;
-		if (options->skyline)
-			world->skylinefile = filedb[j++].file;
-		if (options->writelog)
-			options->logfile = filedb[j++].file;
-		if (options->aic)
-			options->aicfile = filedb[j++].file;
-		if (options->plot && options->plotmethod == PLOTALL)
-			world->mathfile = filedb[j++].file;
+	MYMPIBCAST (tempdb, maxfilenum, MPI_LONG, MASTER, comm_world);
+	filenum = tempdb[0]; 
+	for(i=0, j=1;i<filenum;i++)
+	  {
+	    filedb[i].file = (FILE *) tempdb[j++];
+	    filedb[i].handle =  tempdb[j++];
+	  }
+	//assumess same order as in the master
+	j=1; //zero is stdout
+	world->outfile = filedb[j++].file;
+	if (options->treeprint > 0 && (!options->readsum))
+	  world->treefile = filedb[j++].file;
+	if (options->mighist)
+	  world->mighistfile = filedb[j++].file;
+	if (options->skyline)
+	  world->skylinefile = filedb[j++].file;
+	if (options->writelog)
+	  options->logfile = filedb[j++].file;
+	if (options->aic)
+	  options->aicfile = filedb[j++].file;
+	if (options->plot && options->plotmethod == PLOTALL)
+	  world->mathfile = filedb[j++].file;
         if (options->bayes_infer)
 	  {
 	    if(options->has_bayesfile)
 	      world->bayesfile = filedb[j++].file;
+	    // TEST: experiment with parallele I/O
 	    if(options->has_bayesmdimfile)
+#ifdef PARALIO
+	      MPI_File_open(comm_world,options->bayesmdimfilename,MPI_MODE_CREATE,MPI_INFO_NULL, &world->mpi_bayesmdimfile);
+#else
 	      world->bayesmdimfile = filedb[j++].file;
+#endif
 	  }
 #endif /*MPI*/
-	  }
+      }
     myfree(tempdb);
 }
 
@@ -877,13 +898,13 @@ void
 get_next_word(char **instring, char *delimiters, char **nextword)
 {
   *nextword = strsep(instring, delimiters);
-  while(strchr(delimiters,(*nextword)[0]))
+  while(*nextword != NULL && strchr(delimiters,(*nextword)[0]))
   {
-    if((*instring)==NULL)
-      {
-	(*nextword) = NULL;
-	return;
-      }
+    //    if((*instring)==NULL)
+    //  {
+    //	(*nextword) = NULL;
+    //	return;
+    //      }
       *nextword = strsep(instring,delimiters);
   }
 }
@@ -1535,8 +1556,11 @@ insert_migr_node (world_fmt * world, node * up, node * down,
         error ("up has to be a top-node");
     theNode = showtop (up)->back;
     if (*migr_table_counter > 0 && up->tyme > migr_table[0].time)
+      {
+	printf("%i> up->type=%c tyme=%f showtop(up)time=%f mig[0]tabletyme=%f\n", myID, up->type, up->tyme, showtop(up)->tyme, migr_table[0].time);
         error
-        ("insert_migr_node: the first migration node has a wrong time for up");
+	  ("insert_migr_node: the first migration node has a wrong time for up");
+      }
     if (migr_table[(*migr_table_counter) - 1].from != down->actualpop)
     {
         error ("this should never happen -> wrong choice of nodes\n");
@@ -1553,6 +1577,10 @@ insert_migr_node (world_fmt * world, node * up, node * down,
 			      migr_table[i].to,
 			      migr_table[i].time - theNode->tyme);
     }
+    if(down->tyme < migr_table[i-1].time)
+      {
+	error("Problem with migration addition in coalesce1p()");
+      }
     //printf("%li migration nodelets added\n",(*migr_table_counter) * 2);
     down->back = theNode;
     theNode->back = down;
@@ -3213,11 +3241,15 @@ sgets (char *s, int size, char **stream)
 
 
 /// sticks a text into the printing buffer
-void add_to_buffer(char *fp, long *bufsize, char **buffer)
+void add_to_buffer(char *fp, long *bufsize, char **buffer, long *allocbufsize)
 {
-	(*bufsize) += (long) strlen (fp) + 1;
-	(*buffer) = (char *) myrealloc (*buffer, (*bufsize) * sizeof (char));
-	strcat (*buffer, fp);
+  long fpsize = (long) strlen (fp) + 1;
+  if(*allocbufsize <= (*bufsize + fpsize))
+    {
+      *allocbufsize += 100 * fpsize;
+      (*buffer) = (char *) myrealloc (*buffer, (*allocbufsize) * sizeof (char));
+    }
+  (*bufsize) += sprintf((*buffer) + (*bufsize),"%s",fp);
 }
 
 
